#ifndef ALGORITHM_H
#define ALGORITHM_H
/* Includes ------------------------------------------------------------------*/
#include <math.h>

/* Exported macros -----------------------------------------------------------*/
//外部内存空间加速计算，减少外部读取写入的次数 需要额外ACCELERATE_BUFFER_SIZE * 2 * sizeof(Complex)的空间，以空间换时间
#define ALGORITHM_EXT_MEMORY_ACCELERATE			0
#define ACCELERATE_BUFFER_SIZE					1024
//返回数据结果
#define ALGORITHM_RESULT_TYPE					float
//处理数据类型
#define ALGORITHM_HANDLE_TYPE					float

//一些常用数学定义
#define ALGORITHM_PI							3.141592f
#define ALGORITHM_G								9.80665f

/* Exported types ------------------------------------------------------------*/
typedef struct
{
	float fReal;
	float fImage;
} Complex;

/* Exported functions --------------------------------------------------------*/
ALGORITHM_RESULT_TYPE Mean(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress);
ALGORITHM_RESULT_TYPE RMS(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress);
ALGORITHM_RESULT_TYPE Std(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress);
ALGORITHM_RESULT_TYPE Var(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress);
ALGORITHM_RESULT_TYPE Max(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress);
ALGORITHM_RESULT_TYPE Min(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize, 
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress);
ALGORITHM_RESULT_TYPE Kurtosis(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize, 
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress);
//ALGORITHM_RESULT_TYPE DFT(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize, ALGORITHM_HANDLE_TYPE *pxOut,
//							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress,
//						    void (*pFuncWriteData)(unsigned long, unsigned char *, unsigned long), unsigned long ulWriteAddress);
//ALGORITHM_RESULT_TYPE DFT_GetOnePoint(unsigned long k, ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize,
//							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress);	
ALGORITHM_RESULT_TYPE RFFT(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize, Complex *pxOut,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress,
							void (*pFuncWriteData)(unsigned long, unsigned char *, unsigned long), unsigned long ulWriteAddress);
ALGORITHM_RESULT_TYPE IFFT(Complex *pxData, unsigned long ulSize, ALGORITHM_HANDLE_TYPE *pxOut,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress,
							void (*pFuncWriteData)(unsigned long, unsigned char *, unsigned long), unsigned long ulWriteAddress);
ALGORITHM_RESULT_TYPE GetMaxComplexInFreqRange(Complex *pxFFTResult, unsigned long ulSize, unsigned long ulSamplingRate, float fFrequency, float fRange, 
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress);
ALGORITHM_RESULT_TYPE VRMS(Complex *pxFFTResult, unsigned long ulSize, unsigned long ulSamplingRate,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress);
ALGORITHM_RESULT_TYPE HighFrequencyRMS(Complex *pxFFTResult, unsigned long ulSize, unsigned long ulSamplingRate, float *pxIFFTWave,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress,
							void (*pFuncWriteData)(unsigned long, unsigned char *, unsigned long), unsigned long ulWriteAddress);
ALGORITHM_RESULT_TYPE Add(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize, ALGORITHM_HANDLE_TYPE xOffset, 
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress,
							void (*pFuncWriteData)(unsigned long, unsigned char *, unsigned long), unsigned long ulWriteAddress);
ALGORITHM_RESULT_TYPE CalculateAdaptiveThreshold(ALGORITHM_HANDLE_TYPE *pxData, unsigned long ulSize, 
							ALGORITHM_HANDLE_TYPE *pxHighLimit, ALGORITHM_HANDLE_TYPE *pxLowLimit,
							void (*pFuncReadData)(unsigned long, unsigned char *, unsigned long), unsigned long ulReadAddress, float fScale);
ALGORITHM_RESULT_TYPE Complex_Cal(Complex *pxComplex);
Complex 			  Complex_Add(Complex *pxOp1, Complex *pxOp2);
Complex 			  Complex_Sub(Complex *pxOp1, Complex *pxOp2);
Complex 			  Complex_Mul(Complex *pxOp1, Complex *pxOp2);
#endif

