/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include "modbus_update.h"
#include "flash.h"
#include "pwr_gpio.h"
#include "common.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/timers.h"

/* Private macros ------------------------------------------------------------*/
/* Private functions ---------------------------------------------------------*/
static void MBUpdate_Reset(void);
static void MBUpdate_EraseUpdateArea(void);

/* Private variables ---------------------------------------------------------*/
static unsigned short g_usUpdateIndex;
static unsigned short g_usUpdateBuffer[MBUPDATE_RENEW_PER_SIZE];
static bool g_bReset = false;
static const unsigned short g_usPasswordBuffer[MBUPDATE_CONFIRM_SIZE] = {0x5749, 0x5449, 0x554D}; // WITIUM
static unsigned short g_usInputPassword[MBUPDATE_CONFIRM_SIZE];

/**
 * @brief  update poll
 * @retval None
 */
void MBUpdate_Poll(void)
{
	if (g_bReset == true)
	{
		vTaskDelay(1000);
		MBUpdate_Reset();
	}
}

/**
 * @brief  write to buffer then write to flash
 * @retval None
 */
void MBUpdate_WriteToBufferAndFlash(unsigned short usWriteAddress, unsigned short usWriteValue)
{
	g_usUpdateBuffer[usWriteAddress - MBUPDATE_RENEW_ADDRESS] = usWriteValue;

	if (usWriteAddress == MBUPDATE_RENEW_ADDRESS + MBUPDATE_RENEW_PER_SIZE - 1)
	{
		// Flash_WriteWithoutErase(UPDATE_FLASH_ADDRESS + g_usUpdateIndex * MBUPDATE_RENEW_PER_SIZE * 2, g_usUpdateBuffer, MBUPDATE_RENEW_PER_SIZE);
		g_usUpdateIndex++;
	}
}

/**
 * @brief  get index
 * @retval None
 */
unsigned short MBUpdate_GetIndex(void)
{
	return g_usUpdateIndex;
}

/**
 * @brief  write index
 * @retval None
 */
void MBUpdate_SetIndex(unsigned short usIndexValue)
{
	g_usUpdateIndex = usIndexValue;

	if (usIndexValue == 0x0000)
		MBUpdate_EraseUpdateArea();
}

void MBUpdate_SetPassword(unsigned short usPasswordAddress, unsigned short usPasswordValue)
{
	if (usPasswordAddress == MBUPDATE_CONFIRM_ADDRESS)
	{
		memset(g_usInputPassword, 0, sizeof(g_usInputPassword));
	}

	g_usInputPassword[usPasswordAddress - MBUPDATE_CONFIRM_ADDRESS] = usPasswordValue;

	if (memcmp(g_usInputPassword, g_usPasswordBuffer, sizeof(g_usInputPassword)) == 0)
	{
		// unsigned short usFlagValue = UPDATE_FLAG_VALUE;
		// Password correct
		// Flash_Write(UPDATE_FLAG_ADDRESS, &usFlagValue, sizeof(usFlagValue) / 2);
		g_bReset = true;
	}
}

/**
 * @brief  soft reset
 * @retval None
 */
static void MBUpdate_Reset()
{
	esp_restart();
}

/**
 * @brief  erase update area
 * @retval None
 */
static void MBUpdate_EraseUpdateArea()
{
	// Flash_Erase(UPDATE_FLASH_ADDRESS, UPDATE_FLASH_SIZE * FLASH_1_KB);
}
