/* Includes ------------------------------------------------------------------*/
#include "quectel_file.h"

/**
  * @brief  Quectel file open
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_FileOpen(char *pcFileName, unsigned char ucMode, unsigned long *pulFileHandle)
{
	Celluar_ErrorStatus eResult;
	char cCommand[48], cResponse[48];
	
	if (pcFileName == NULL || strlen(pcFileName) > 20)
		return Celluar_Error;
	
	Celluar_Take();
	sprintf(cCommand, "AT+QFOPEN=\"%s\",%d\r", pcFileName, ucMode);
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+QFOPEN:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_1S)) == Celluar_OK)
	{
		char *pcFileHandlePointer = NULL;
		
		pcFileHandlePointer = strstr(cResponse, "+QFOPEN: ");
		
		if (pcFileHandlePointer != NULL)
		{
			pcFileHandlePointer += strlen("+QFOPEN: ");
			if (pulFileHandle != NULL)
				*pulFileHandle = atol(pcFileHandlePointer);
		}
		else
		{
			eResult = Celluar_Error;
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel file seek
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_FileSeek(unsigned long ulFileHandle, unsigned long ulOffset)
{
	Celluar_ErrorStatus eResult;
	char cCommand[48];
	
	Celluar_Take();
	sprintf(cCommand, "AT+QFSEEK=%lu,%lu\r", ulFileHandle, ulOffset);
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_1S);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel file read
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_FileRead(unsigned long ulFileHandle, unsigned long ulLength, unsigned char *pucBuffer, unsigned long ulBufferLen, unsigned long *pulRecvLength)
{
	Celluar_ErrorStatus eResult;
	char cCommand[48];
	unsigned long ulRecvBufferLen;
	
	Celluar_Take();
	if (ulLength == 0)
		sprintf(cCommand, "AT+QFREAD=%lu\r", ulFileHandle);
	else
		sprintf(cCommand, "AT+QFREAD=%lu,%lu\r", ulFileHandle, ulLength);
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "CONNECT", (char *)pucBuffer, ulBufferLen, &ulRecvBufferLen, CELLUAR_WAITTIME_1S)) == Celluar_OK)
	{	
		char *pcLenPointer = NULL;
		char *pcDataPointer = NULL;
		
		pcLenPointer = strstr((char *)pucBuffer, "CONNECT");
		pcDataPointer = strstr((char *)pcLenPointer, "\r\n");
		
		if (pcLenPointer != NULL && pcDataPointer != NULL)
		{
			unsigned long ulLen;
			unsigned long ulRecvDataLen;
			
			pcLenPointer += strlen("CONNECT");
			pcDataPointer += strlen("\r\n");

			ulLen = atol(pcLenPointer);
			if (pulRecvLength != NULL)
				*pulRecvLength = ulLen;
			ulRecvDataLen = ulRecvBufferLen - (pcDataPointer - (char *)pucBuffer);
			if (ulRecvDataLen >= ulLen)
			{
				memmove(pucBuffer, pcDataPointer, ulLen);
			}
			else
			{
				memset(pucBuffer, 0, ulBufferLen);
				eResult = Celluar_Error;
			}
		}
		else
		{
			eResult = Celluar_Error;
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel file close
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_FileClose(unsigned long ulFileHandle)
{
	Celluar_ErrorStatus eResult;
	char cCommand[48];
	
	Celluar_Take();
	sprintf(cCommand, "AT+QFCLOSE=%lu\r", ulFileHandle);
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, 0, NULL, CELLUAR_WAITTIME_1S);
	Celluar_Give();
	return eResult;
}

