/* Includes ------------------------------------------------------------------*/
#include "quectel_gnss.h"
#include "quectel_3gpp.h"

#if QUECTEL > 0
#if CELLUAR_GNSS_ENABLE > 0
/**
  * @brief  Quectel set gnss power status
  * @param  None
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus	Quectel_ControlGNSSPower(bool bPowerOn)
{	
	Celluar_ErrorStatus eResult;
	char cCommand[50];
	
	Celluar_Take();
	sprintf(cCommand, "AT+QLBSCFG=\"token\",\"64Y2dJ682r35YuQ2\"\r");
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, NULL, NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();
	
	Celluar_Take();
	sprintf(cCommand, "AT+QLBSCFG?\r");
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, NULL, NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();	
	return eResult;	
	
	
}

/**
  * @brief  Quectel get longitude and latitude
  * @param  None
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus		Quectel_GetGNSSNavigationInfomation(double *pdLongitude, double *pdLatitude)
{
    Quectel_ControlGNSSPower(true);
	Celluar_ErrorStatus eResult;
	char cCommand[16], cResponse[200];
	
	Celluar_Take();
	sprintf(cCommand, "AT+QLBS\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+QLBS:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_3M)) == Celluar_OK)
	{
		char *pcTemp = NULL;
		char *pcSplitedData[3] = {NULL, NULL, NULL};
		
		pcTemp = strstr(cResponse, "+QLBS:") + strlen("+QLBS:");
		Split(pcSplitedData, 3, pcTemp, ",");
		
		//Get longitude and latitude
		if (pcSplitedData[1] && pcSplitedData[2])
		{
			if (pdLatitude != NULL)
				*pdLatitude 	= atof(pcSplitedData[1]);
			if (pdLongitude != NULL)
				*pdLongitude 	= atof(pcSplitedData[2]);
			
			if (*pdLatitude == 0 && *pdLatitude == 0)
				eResult = Celluar_Error;
		}
		else
		{
			eResult = Celluar_Error;
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel get cell array info
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus	Quectel_GetCellArrayInfo(unsigned long *pulMcc, unsigned long *pulMnc, unsigned long *pulLac, unsigned long *pulCi, unsigned long *pulSignal, unsigned long *pulCellCount)
{
	unsigned long ulMccMnc = 0;
	
	*pulCellCount = 0;
	if (Quectel_GetMccMnc(&ulMccMnc) == Celluar_OK)
	{
		pulMcc[0] = ulMccMnc / 100;
		pulMnc[0] = ulMccMnc - (ulMccMnc / 100) * 100;
		
		if (pulMcc[0] != 0)
		{
			if (Quectel_GetLacCi(&pulLac[0], &pulCi[0]) == Celluar_OK)
			{
				unsigned char ucSignal = 0;
				if (Quectel_GetSignal(&ucSignal, NULL) == Celluar_OK)
				{
					//unkonwn set to the 0
					if (ucSignal == 99)
						ucSignal = 0;
					pulSignal[0] = ucSignal + 110;
					(*pulCellCount)++;
					return Celluar_OK;
				}
			}
		}
	}
	
	return Celluar_Error;
}

/**
  * @brief  Quectel get mcc mnc
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_GetMccMnc(unsigned long *pulMccMnc)
{
	Celluar_ErrorStatus eResult;
	char cCommand[16], cResponse[48];
	
	Celluar_Take();
	sprintf(cCommand, "AT+COPS?\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+COPS:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
	{
		char *pcSplitedData[2] = {NULL, NULL};
		
		Split(pcSplitedData, 2, cResponse, "\"");
		if (pcSplitedData[1] != NULL)
		{
			*pulMccMnc = atol(pcSplitedData[1]);
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  Quectel get lac ci
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus Quectel_GetLacCi(unsigned long *pulLac, unsigned long *pulCi)
{
	Celluar_ErrorStatus eResult;
	char cCommand[16], cResponse[48];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CREG?\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+CREG:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
	{
		char *pcSplitedData[4] = {NULL, NULL, NULL};
		
		Split(pcSplitedData, 4, cResponse, "\"");
		if (pcSplitedData[1] != NULL && pcSplitedData[3] != NULL)
		{
			*pulLac = strtol(pcSplitedData[1], NULL, 16);
			*pulCi = strtol(pcSplitedData[3], NULL, 16);
		}
	}
	Celluar_Give();
	return eResult;
}
#endif
#endif
