/* Includes ------------------------------------------------------------------*/
#include "simcom.h"
#include "celluar_comm.h"
#include "celluar_event.h"
#include "common.h"

#if SIMCOM > 0
/* Private macros ------------------------------------------------------------*/
#if CELLUAR_MULTIIPCONNECTION_ENABLE > 0
	#define SIMCOM_NET_RECEIVED_MESSAGE			"+RECEIVE"
#else
	#define SIMCOM_NET_RECEIVED_MESSAGE			"+IPD"
#endif
#define SIMCOM_NET_CLOSED_MESSAGE				"CLOSED"
#define SIMCOM_POWER_DOWN_MESSAGE				"NORMAL POWER DOWN"
#define SIMCOM_PULL_CARD_MESSAGE				"+CPIN: NOT READY"
#define SIMCOM_PDP_DEACT_MESSAGE				"+PDP: DEACT"
#define SIMCOM_HTTP_READ_MESSAGE				"+HTTPREAD:"

/* Private variables ---------------------------------------------------------*/ 
/* Extern variables ----------------------------------------------------------*/ 
extern Celluar g_xCelluar;

/* Private functions ---------------------------------------------------------*/ 
#if CELLUAR_TCPIP_ENABLE > 0
static long 	SIMCom_SearchForAPN(void);
static bool 	SIMCom_ConfigAPN(char *pcAPN, char *pcUser, char *pcPassword);
#endif

/**
  * @brief  SIMCom poll
  * @param  None
  * @retval None
  */
void SIMCom_Poll()
{
	Celluar_Buffer *pxBuffer;
	xSemaphoreTake(g_xCelluar.xReceivedSemHandle, portMAX_DELAY);
	
	pxBuffer = &g_xCelluar.xReceiveQueue.xReceiveBuffer[(((signed long)g_xCelluar.xReceiveQueue.ulRear - 1) + CELLUAR_QUEUE_SIZE) % CELLUAR_QUEUE_SIZE];
	
#if DEBUG > 0
	Debug_Print("\r\n");
	Debug_PrintData(pxBuffer->ucBuffer, pxBuffer->ulBufferLen);
#endif
	
	if(memstr(pxBuffer->ucBuffer, SIMCOM_NET_RECEIVED_MESSAGE, pxBuffer->ulBufferLen))
	{
		unsigned char *pucBufferPointer = pxBuffer->ucBuffer;
		unsigned long ulRestHandlerLen = pxBuffer->ulBufferLen;
		
		while ((pucBufferPointer = (unsigned char *)memstr(pucBufferPointer, SIMCOM_NET_RECEIVED_MESSAGE, ulRestHandlerLen)) != NULL)
		{
			unsigned char ucSocketID 	= 0;
			unsigned char *pucMsgStart;
			unsigned char *pucDataLen;
			unsigned char *pucData;
			unsigned long ulDataLen;
			
			//Socket ID | Data  // ulDataLen = 1(socket id) + len(data)
		#if CELLUAR_MULTIIPCONNECTION_ENABLE > 0
			unsigned char *pucSocketNum;
			
			pucMsgStart 	= (unsigned char *)strstr((char *)pucBufferPointer, "+RECEIVE,");
			pucSocketNum 	= pucMsgStart + strlen("+RECEIVE,");
			ucSocketID 		= atol((char *)pucSocketNum);
			pucDataLen 		= pucSocketNum + 2;
		#else
			pucMsgStart 	= (unsigned char *)strstr((char *)pucBufferPointer, "+IPD,");
			pucDataLen 		= pucMsgStart + strlen("+IPD,");
		#endif
			pucData 		= (unsigned char *)strstr((char *)pucMsgStart, ":");
			
			if (pucMsgStart != NULL && pucData != NULL)
			{
				unsigned long ulRestDataLength = 0;
				
				*pucData 			= ucSocketID;
				ulDataLen 			= atol((char *)pucDataLen);
				ulRestDataLength 	= pxBuffer->ucBuffer + pxBuffer->ulBufferLen - 1 - pucData;
				if (ulDataLen <= ulRestDataLength)
					Celluar_EventPost(Celluar_EventNetReceived, pucData, ulDataLen + 1);
				else		//error occured
					break;
				
				//set offset
				pucBufferPointer = pucData + 1 + ulDataLen;
				//clear receive data to ' ' (space)
				memset(pucMsgStart, 32, pucBufferPointer - pucMsgStart);
				//cal rest length
				ulRestHandlerLen = pxBuffer->ucBuffer + pxBuffer->ulBufferLen - pucBufferPointer;
			}
			else
			{
				//error occured
				break;
			}
		}
	}
	else if (memstr(pxBuffer->ucBuffer, SIMCOM_HTTP_READ_MESSAGE, pxBuffer->ulBufferLen))
	{
		//NULL
	}
	else if(memstr(pxBuffer->ucBuffer, SIMCOM_NET_CLOSED_MESSAGE, pxBuffer->ulBufferLen))
	{
		unsigned char ucClosedSocketID = atol((char *)pxBuffer->ucBuffer);
		Celluar_EventPost(Celluar_EventNetClosed, &ucClosedSocketID, sizeof(ucClosedSocketID));
	}
	else if (memstr(pxBuffer->ucBuffer, SIMCOM_POWER_DOWN_MESSAGE, pxBuffer->ulBufferLen))
	{
		Celluar_EventPost(Celluar_EventPowerDown, NULL, 0);
	}
	else if (memstr(pxBuffer->ucBuffer, SIMCOM_PULL_CARD_MESSAGE, pxBuffer->ulBufferLen))
	{
		Celluar_EventPost(Celluar_EventPullCard, NULL, 0);
	}
	else if (memstr(pxBuffer->ucBuffer, SIMCOM_PDP_DEACT_MESSAGE, pxBuffer->ulBufferLen))
	{
		Celluar_EventPost(Celluar_EventPDPDeact, NULL, 0);
	}
	
	if (g_xCelluar.bWaiting)
		xSemaphoreGive(g_xCelluar.xResponseSemHandle);
	else
		Celluar_QueuePop(&g_xCelluar.xReceiveQueue);
}

/**
  * @brief  SIMCom TCP/IP Init
  * @param  None
  * @retval None
  */
void SIMCom_BasicSettings()
{
#if CELLUAR_GNSS_ENABLE > 0
	//开启GPS
	SIMCom_ControlGNSSPower(true);
	//设置成工程师模式，获取多基站信息
	SIMCom_SwitchToEngineeringMode(3, 1);
#endif
#if CELLUAR_TCPIP_ENABLE > 0
	//显示LAC CI 基站信息
	SIMCom_SetNetworkRegistrationMode(2);
	//显示国家信息
	SIMCom_SetCOPSMode(0, 2);
	//TCPIP收到数据后显示IPD前缀
	SIMCom_SetIPHead(true);
#if CELLUAR_USE_SSL > 0
    //设置SSL
    SIMCom_SetSSL(true);
#endif
#endif
}

#if CELLUAR_TCPIP_ENABLE > 0
/**
  * @brief  SIMCom TCP/IP Init
  * @param  None
  * @retval None
  */
void SIMCom_TCPIPInit(unsigned long *pulRecommendedAPNIndex)
{
	if (g_xCelluar.bIsPowerOn)
	{
		//检查卡
		if (SIMCom_GetCardStatus() == Celluar_OK)
		{
			g_xCelluar.bHasSIMCard = true;
		}
		else
		{
			//无卡
			g_xCelluar.bHasSIMCard = false;
		}
		
		//检查到SIM卡后进行网络设置
		if (g_xCelluar.bHasSIMCard)
		{
			unsigned char ucWaitTime = CELLUAR_TRYCOUNT_90;
			bool bRegisted = false;
			bool bNetAttached = false;
			
			//等待网络 90s超时
			while (ucWaitTime && g_xCelluar.bHasSIMCard)
			{
				//等待网络注册
				if (SIMCom_GetRegistrationStatus() == Celluar_OK)
				{
					bRegisted = true;
					break;
				}
				ucWaitTime--;
				vTaskDelay(1000);
			}
			
			ucWaitTime = CELLUAR_TRYCOUNT_90;
			while (bRegisted && ucWaitTime && g_xCelluar.bHasSIMCard)
			{
				//等待网络附着
				if (SIMCom_GetNetworkAttachStatus() == Celluar_OK)
				{
					bNetAttached = true;
					break;
				}
				ucWaitTime--;
				vTaskDelay(1000);
			}
			
			if (bNetAttached)
			{
				//检查网络状态
				char cIPStatus[48];
				if (SIMCom_GetNetStatus(cIPStatus, sizeof(cIPStatus)) == Celluar_OK)
				{
					//网络已连接或者处于可连接状态 直接返回
					if (!strstr(cIPStatus, "IP INITIAL") && !strstr(cIPStatus, "PDP DEACT"))
					{
						g_xCelluar.bHasNetwork = true;
					}
					else
					{
						//设置APN时 需要IP INITIAL状态
						if (!strstr(cIPStatus, "IP INITIAL"))
						{
							if (SIMCom_DeactivateGPRS() != Celluar_OK)
							{
								//Error
								return;
							}
						}
							
						#if CELLUAR_MULTIIPCONNECTION_ENABLE > 0
							//只有在IP INITIAL 状态下可以打开多连接
							SIMCom_StartUpMultiIPConnection(true);
						#endif
					}
				}
				
				//网络附着成功，设置推荐APN，如果失败搜寻APN 在g_pcAPNList
				if (g_xCelluar.bHasNetwork == false && pulRecommendedAPNIndex != NULL && *pulRecommendedAPNIndex < CELLUAR_APN_COUNT)
				{
					if (SIMCom_ConfigAPN(g_pcAPNList[*pulRecommendedAPNIndex], "", "") == true)
					{
						g_xCelluar.ulCurrentAPNIndex = *pulRecommendedAPNIndex;
						g_xCelluar.bHasNetwork = true;
					}
				}
					
				//推荐APN不成功，搜寻APN列表
				if (g_xCelluar.bHasNetwork == false)
				{
					long lTemp;
					if ((lTemp = SIMCom_SearchForAPN()) != -1)
					{
						if (pulRecommendedAPNIndex != NULL)
							*pulRecommendedAPNIndex = lTemp;
						g_xCelluar.ulCurrentAPNIndex = lTemp;
						g_xCelluar.bHasNetwork = true;
					}
				}
			}
		}	
	}
}

#if CELLUAR_NTP_ENABLE > 0
bool SIMCom_SyncNetworkTime(Calendar *pxCalendar)
{
	int i = 0;
	//ntp basic settings
	SIMCom_BearerSettrings(3, 1, "Contype", "GPRS");
	SIMCom_BearerSettrings(3, 1, "APN", g_pcAPNList[g_xCelluar.ulCurrentAPNIndex]);
	SIMCom_BearerSettrings(1, 1, NULL, NULL);
	SIMCom_SetGPRSBearerID(1);
	
	//sync
	for (i = 0;i < CELLUAR_NTP_SERVER_COUNT;i++)
	{
		SIMCom_SetNTPServerAndTimeZone(g_pcNTPServerList[i], 0);
		
		if (SIMCom_SynchronizeNetworkTime() == Celluar_OK)
		{
			if (SIMCom_GetLocalTime(pxCalendar) == Celluar_OK)
			{
				return true;
			}
		}
	}

	return false;
}
#endif

#if CELLUAR_TCPIP_ENABLE > 0
/**
  * @brief  SIMCom search apn in buffer
  * @param  None
  * @retval success apn index (-1 error)
  */
static long 	SIMCom_SearchForAPN()
{
	unsigned long ulIndex;
	for (ulIndex = 0;ulIndex < CELLUAR_APN_COUNT;ulIndex++)
	{
		if (SIMCom_DeactivateGPRS() == Celluar_OK)
		{
			if (SIMCom_ConfigAPN(g_pcAPNList[ulIndex], "", ""))
			{
				return ulIndex;
			}
		}
		else
		{
			break;
		}
	}
	
	return -1;
}

/**
  * @brief  SIMCom config apn
  * @param  None
  * @retval true / false
  */
static bool 	SIMCom_ConfigAPN(char *pcAPN, char *pcUser, char *pcPassword)
{	
	if (SIMCom_SetAPN(pcAPN, pcUser, pcPassword) == Celluar_OK)
	{
		if (SIMCom_BringUpWirelessConnection() == Celluar_OK)
		{
			if (SIMCom_GetLocalIPAddress() == Celluar_OK)
			{
				return true;
			}
		}
	}
	
	return false;
}
#endif

#if CELLUAR_HTTP_ENABLE > 0
/**
  * @brief  SIMCom http post (if response is big data, please set pucResponse null and get data 1k per times after post)
  * @param  None
  * @retval true / false
  */
bool SIMCom_HTTPPost(char *pcURL, unsigned char *pucPostData, unsigned long ulPostLen, unsigned char *pucResponse, unsigned long ulBufferLen, unsigned long *pulRespLen, unsigned long *pulErr)
{
	unsigned long ulResponseLen = 0;
	bool eState = false;
	
	if (pcURL == NULL || pucPostData == NULL)
		return eState;
	
	//Bearer config
	SIMCom_BearerSettrings(3, 1, "Contype", "GPRS");
	SIMCom_BearerSettrings(3, 1, "APN", g_pcAPNList[g_xCelluar.ulCurrentAPNIndex]);
	SIMCom_BearerSettrings(1, 1, NULL, NULL);
	SIMCom_SetGPRSBearerID(1);
	
	//Post
	SIMCom_InitializeHTTPService();
	SIMCom_SetHTTPParametersValue("CID", NULL, 1);
	SIMCom_SetHTTPParametersValue("URL", pcURL, NULL);
	SIMCom_InputHTTPData(pucPostData, ulPostLen, 10000);
	if (SIMCom_HTTPAction(POST, &ulResponseLen, pulErr) == Celluar_OK)
	{
		eState = true;
	}
	
	if (pulRespLen != NULL)
		*pulRespLen = ulResponseLen;
	if (ulResponseLen + CELLUAR_HTTP_RESPONSE_HEAD_SIZE < ulBufferLen && pucResponse != NULL && ulResponseLen != 0)
	{
		SIMCom_ReadHTTPServerResponse(0, 0, pucResponse, ulBufferLen, NULL);
		SIMCom_TerminateHTTPService();
	}
	else if (ulResponseLen == 0 || pucResponse != NULL)
	{
		SIMCom_TerminateHTTPService();
	}
	
	return eState;
}

/**
  * @brief  SIMCom http get (if response is big data, please set pucResponse null and get data 1k per times after get)
  * @param  None
  * @retval true / false
  */
bool SIMCom_HTTPGet(char *pcURL, unsigned char *pucResponse, unsigned long ulBufferLen, unsigned long *pulRespLen, unsigned long *pulErr)
{
	unsigned long ulResponseLen = 0;
	bool eState = false;
	
	if (pcURL == NULL)
		return eState;
	
	//Bearer config
	SIMCom_BearerSettrings(3, 1, "Contype", "GPRS");
	SIMCom_BearerSettrings(3, 1, "APN", g_pcAPNList[g_xCelluar.ulCurrentAPNIndex]);
	SIMCom_BearerSettrings(1, 1, NULL, NULL);
	SIMCom_SetGPRSBearerID(1);
	
	//Post
	SIMCom_InitializeHTTPService();
	SIMCom_SetHTTPParametersValue("CID", NULL, 1);
	SIMCom_SetHTTPParametersValue("URL", pcURL, NULL);
	if (SIMCom_HTTPAction(GET, &ulResponseLen, pulErr) == Celluar_OK)
	{
		eState = true;
	}
	
	if (pulRespLen != NULL)
		*pulRespLen = ulResponseLen;
	if (ulResponseLen < ulBufferLen && pucResponse != NULL && ulResponseLen != 0)
	{
		SIMCom_ReadHTTPServerResponse(0, 0, pucResponse, ulBufferLen, NULL);
		SIMCom_TerminateHTTPService();
	}
	else if (ulResponseLen == 0 || pucResponse != NULL)
	{
		SIMCom_TerminateHTTPService();
	}
	
	return eState;
}
#endif

#endif

#endif
