/* Includes ------------------------------------------------------------------*/
#include "simcom_gnss.h"

#if SIMCOM > 0
#if CELLUAR_GNSS_ENABLE > 0
/**
  * @brief  SIMCom set gnss power status
  * @param  None
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus		SIMCom_ControlGNSSPower(bool bPowerOn)
{
	Celluar_ErrorStatus eResult;
	char cCommand[16];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CGNSPWR=%d\r", bPowerOn ? 1 : 0);
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, NULL, NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  SIMCom get longitude and latitude
  * @param  None
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus		SIMCom_GetGNSSNavigationInfomation(double *pdLongitude, double *pdLatitude)
{
	Celluar_ErrorStatus eResult;
	char cCommand[16], cResponse[200];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CGNSINF\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "+CGNSINF:", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
	{
		char *pcTemp = NULL;
		char *pcSplitedData[5] = {NULL, NULL, NULL, NULL, NULL};
		
		pcTemp = strstr(cResponse, "+CGNSINF:") + strlen("+CGNSINF:");
		Split(pcSplitedData, 5, pcTemp, ",");
		
		//Get longitude and latitude
		if (pcSplitedData[3] && pcSplitedData[4])
		{
			if (pdLatitude != NULL)
				*pdLatitude 	= atof(pcSplitedData[3]);
			if (pdLongitude != NULL)
				*pdLongitude 	= atof(pcSplitedData[4]);
			
			if (*pdLatitude == 0 && *pdLatitude == 0)
				eResult = Celluar_Error;
		}
		else
		{
			eResult = Celluar_Error;
		}
	}
	Celluar_Give();
	return eResult;
}

/**
  * @brief  SIMCom enter into engineering mode 3 1
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus	SIMCom_SwitchToEngineeringMode(unsigned char ucMode, unsigned char ucNcell)
{
	Celluar_ErrorStatus eResult;
	char cCommand[16];
	
	Celluar_Take();
	sprintf(cCommand, "AT+CENG=%d,%d\r", ucMode, ucNcell);
	eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", NULL, NULL, NULL, CELLUAR_WAITTIME_500MS);
	Celluar_Give();
	return eResult;
}

/**
  * @brief  SIMCom get cell array info
  * @retval Celluar_ErrorStatus
  */
Celluar_ErrorStatus	SIMCom_GetCellArrayInfo(unsigned long *pulMcc, unsigned long *pulMnc, unsigned long *pulLac, unsigned long *pulCi, unsigned long *pulSignal, unsigned long *pulCellCount)
{
	Celluar_ErrorStatus eResult;
	char cCommand[12], cResponse[320] = {0};
	
	Celluar_Take();
	*pulCellCount = 0;
	sprintf(cCommand, "AT+CENG?\r");
	if ((eResult = Celluar_SendCommandAndWaitForResponse(cCommand, strlen(cCommand), "OK", cResponse, sizeof(cResponse), NULL, CELLUAR_WAITTIME_500MS)) == Celluar_OK)
	{
		int i;
		char cCENGx[12];
		
		//Max cell count is 7
		for (i = 6;i >= 0;i--)
		{
			char *pcSplitedData[2] = {NULL, NULL};
			
			sprintf(cCENGx, "+CENG: %d,\"", i);
			Split(pcSplitedData, 2, cResponse, cCENGx);
			if (pcSplitedData[1] != NULL && *pcSplitedData[1] != ',')
			{
				char *pcCellSplitedData[6] = {NULL, NULL, NULL, NULL, NULL, NULL};
				
				Split(pcCellSplitedData, 6, pcSplitedData[1], ",");
				if (pcCellSplitedData[0] && pcCellSplitedData[1] && pcCellSplitedData[2] && pcCellSplitedData[3] &&
					pcCellSplitedData[4] && pcCellSplitedData[5])
				{
					if (pulMcc != NULL)
						pulMcc[*pulCellCount] 		= atol(pcCellSplitedData[0]);
					if (pulMnc != NULL)
						pulMnc[*pulCellCount] 		= atol(pcCellSplitedData[1]);
					if (pulLac != NULL)
						pulLac[*pulCellCount] 		= strtol(pcCellSplitedData[2], NULL, 16);
					if (pulCi != NULL)
						pulCi[*pulCellCount] 		= strtol(pcCellSplitedData[3], NULL, 16);
					if (pulSignal != NULL)
						pulSignal[*pulCellCount] 	= atol(pcCellSplitedData[5]) + 110;
					//check
					if (pulMcc[*pulCellCount] != 0 || pulCellCount != NULL)
						(*pulCellCount)++;
				}
			}
		}
		
		if (pulCellCount==NULL || (*pulCellCount) == 0)
			eResult = Celluar_Error;
	}
	Celluar_Give();
	return eResult;
}
#endif
#endif
