#include "wcp.h"
#include <string.h>
#include <stdint.h>
#include "esp_log.h"
#include "wcpCallBack.h"
#include "ble_client.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/event_groups.h"

typedef struct
{
	eWCPinfo     (*pFuncCallBack)(unsigned char *, unsigned short, unsigned short, eWCPoperate);
    bool         (*chrCB[BLE_CHR_HANDLER_NUM])(int, void *, size_t);
} sWCP_Handler;
 
#define WCP_CREATE_SIGNAL(signal)		signal = xSemaphoreCreateBinary()
#define WCP_TAKE(signal)				xSemaphoreTake(signal, portMAX_DELAY)
#define WCP_GIVE(signal)				xSemaphoreGive(signal)

static eTies                  eTi;
static char                   send_data_buff[WCP_MAX_NUM];
static sBuf                   s_recv_buf;
static sWCP_Handler           gWCPHandler;
static SemaphoreHandle_t      gWCPReadSemaphore;
static SemaphoreHandle_t      gWCPWriteSemaphore;
static SemaphoreHandle_t      gWCPCBSemaphore;
static EventGroupHandle_t     eventGroup;
static data_frame             gDtfa = {
	.code          = 0xFFFF,
    .current_id    = 0x0000,
    .total_pack_id = 0x0000,
    .data_size     = 0x0000,
    .data          = NULL,
};

static void     wcp_read_task(void *param);
static eWCPinfo eWcpGetData(unsigned int usCode, unsigned int usSize, unsigned char *usData);
static void     wcp_data_operation(const unsigned short ucWCPtype, const char *ucData, unsigned short usSize);
bool g_Flag = false;
// 增加特性属性
void wcp_init(eTies eis)
{
	WCP_CREATE_SIGNAL(gWCPReadSemaphore);
	WCP_GIVE(gWCPReadSemaphore);
	WCP_CREATE_SIGNAL(gWCPWriteSemaphore);
	WCP_GIVE(gWCPWriteSemaphore);
	WCP_CREATE_SIGNAL(gWCPCBSemaphore);
	WCP_GIVE(gWCPCBSemaphore);
	eTi = eis;
	switch(eis)
	{
		case APKG_SALVE: 
			gWCPHandler.pFuncCallBack                   = eWCPAddressCB;
		break;
		default: break;
	}
	gWCPHandler.chrCB[SENSOR_CHR_WRITE_HANDLER]       = ble_sensor_write;
    eventGroup = xEventGroupCreate();
    xTaskCreate(wcp_read_task, "wcp_read", 4096, NULL, 5, NULL);
}

/**
 *     apkg包任务
 **/
static void wcp_read_task(void *param)
{
	while(1)
	{   
		// EventBits_t eventBits = 
		xEventGroupWaitBits(eventGroup, APKG_EVENT_BIT, pdTRUE, pdFALSE, portMAX_DELAY);
		wcp_data_operation(s_recv_buf.usType, (const char *)s_recv_buf.buf, s_recv_buf.size);
	}
}

/**
 *                          数据搬移操作               
 * @param    usType         操作的回调类型
 * @param    buf            ble协议栈获取的数据
 * @param    size           获取数据的大小
 **/
void wcp_data_migration(unsigned int uiIndex, unsigned short usType, unsigned char *buf, unsigned short size)
{
	WCP_TAKE(gWCPCBSemaphore);
	memset(&s_recv_buf, 0, sizeof(s_recv_buf));
	s_recv_buf.uiDevice = uiIndex;
    s_recv_buf.usType   = usType;
	s_recv_buf.size     = size;
	// esp_log_buffer_hex("cm", buf, size);
	memcpy(s_recv_buf.buf, buf, size);	
	wcp_set_event_bit();
}

/**
 *                          设置事件标志位              
 **/
void wcp_set_event_bit(void)
{
	xEventGroupSetBits(eventGroup, APKG_EVENT_BIT);
}

/**
 *                             根据地址和大小访问外围设备数据               
 * @param    ucWCPtype         操作的回调类型
 * @param    usStartCode       获取的起始地址
 * @param    usAddrNumber      获取数据的大小
 * @param    usData            获取的数据
 * @return   eWCPinfo          得到数据的信息
 **/
eWCPinfo  eWCPReadDataAddress(int index, const unsigned short ucWCPtype, const unsigned int usStartCode, const unsigned int usAddrNumber, unsigned char *usData)
{
	eWCPinfo eWCP = WCP_OK;
	unsigned int u32SendStartTick = 0;
	char *af = NULL;
	WCP_TAKE(gWCPReadSemaphore);	
    if(ucWCPtype >= BLE_CHR_HANDLER_NUM) {
		eWCP = WCP_OTHER_ERROR;	
	}
	af = fsad_get_request_pack(usStartCode, usAddrNumber);	
	if(gWCPHandler.chrCB[ucWCPtype] != NULL){
		u32SendStartTick = xTaskGetTickCount();
		gWCPHandler.chrCB[ucWCPtype](index, af, sizeof(access_frame) - 2);	
	}
	else {
		eWCP = WCP_NO_REG_CB;	
	}
	while(1)
	{   
		if(eWcpGetData(usStartCode, usAddrNumber, usData) == WCP_OK) {
			eWCP = WCP_OK;
			break;
		}
		if (abs((long long)xTaskGetTickCount() - (long long)u32SendStartTick) > APKG_TIME_OUT)
		{
			eWCP = WCP_TIME_OUT;
			break;
		}
		vTaskDelay(1 / portTICK_PERIOD_MS);
	}
	WCP_GIVE(gWCPReadSemaphore);
	return eWCP;
}

/**
 *                             根据地址和大小写入外围设备数据               
 * @param    ucWCPtype         操作的回调类型
 * @param    usStartCode       获取的起始地址
 * @param    usAddrNumber      获取数据的大小
 * @param    usData            写入的数据
 * @return   eWCPinfo          写入的数据信息
 **/
eWCPinfo   eWCPWriteDataAddress(int index, const unsigned short ucWCPtype, unsigned int usStartCode, const unsigned int usAddrNumber, void* vData)
{
	if(ucWCPtype >= BLE_CHR_HANDLER_NUM) {
		return WCP_OTHER_ERROR;	
	}
	WCP_TAKE(gWCPWriteSemaphore);	
	eWCPinfo rc = WCP_OK;
	unsigned int sz = 0;
	unsigned int total_id = 0;
	unsigned int cnt = 0;
	char *buf = NULL;
	unsigned int size = 0;
	unsigned int u32SendStartTick = 0;
	for(int i = 0; i < usAddrNumber; i += APKG_UNSIGNED_CHAR_MAX_NUM) {
		sz = (usAddrNumber - i) > APKG_UNSIGNED_CHAR_MAX_NUM ? APKG_UNSIGNED_CHAR_MAX_NUM : (usAddrNumber - i);
		buf = fsad_get_data_pack(usStartCode, cnt++, total_id, sz, (char *)vData, &size);
		// esp_log_buffer_hex("cm", buf, size);  
		u32SendStartTick = xTaskGetTickCount();
		while(1)
		{
			if(gWCPHandler.chrCB[ucWCPtype](index, buf, size) == true) {
				vTaskDelay(50 / portTICK_PERIOD_MS);
				break;			
			}
			// 内存不足超时判断
			if (abs((long long)xTaskGetTickCount() - (long long)u32SendStartTick) > APKG_FREE_SIZE_TIME_OUT)
			{
				ESP_LOGD("apkg","free_Size time out");
                rc = WCP_TIME_OUT;
				break;
			}
			vTaskDelay(20 / portTICK_PERIOD_MS);	
		}		
	}	
	WCP_GIVE(gWCPWriteSemaphore);
	return rc;
}

/**
 *                             在任务中操作获取的数据           
 * @param    ucWCPtype         操作的回调类型
 * @param    ucData            操作的数据
 * @param    usSize            操作的数据大小
 **/
static void  wcp_data_operation(const unsigned short ucWCPtype, const char *ucData, unsigned short usSize)
{
    eWCPoperate eOp;
	eWCPinfo    efi;
	eOp = fsad_confirm_the_frame_type(ucData);
    uint8_t count;
	switch(eOp)
	{
		case ADDR_WRITE:  
		    data_frame df;   
			efi = fsad_data_frame_parsing(ucData, usSize, &df);
			if(efi != WCP_OK) {
				ESP_LOGD("apkg","parsing fail");
			}
			switch(eTi) {
				case APKG_MASTER: 
					gDtfa.code          = df.code;
					gDtfa.data_size     = df.data_size;	
                    gDtfa.data          = df.data;
					gDtfa.current_id    = df.current_id;
					gDtfa.total_pack_id = df.total_pack_id;
					gDtfa.crc           = df.crc;
					gDtfa.data          = df.data;
                    
                    count = 0;
                    g_Flag = false;
                    while(1)
                    {   
                        count++;
                        vTaskDelay(1 / portTICK_PERIOD_MS);
                        
                        if(g_Flag == true)
                        {
                            break;
                        }
                        if(count > 10)
                        {
                            break;
                        }
                    }
				break;
				case APKG_SALVE:
					if(efi == WCP_OK && gWCPHandler.pFuncCallBack != NULL) {
						gWCPHandler.pFuncCallBack((unsigned char *)df.data, df.code  + (df.current_id * gDtfa.data_size) , df.data_size, ADDR_WRITE);
					}
				break;
			}
		break;    
		case ADDR_READ:
		    unsigned short sz = 0;
		    access_frame af;
			unsigned int total_id = 0;
			unsigned int cnt = 0;
			char *buf = NULL;
			unsigned int size = 0;
			unsigned int u32SendStartTick = 0;
			bool bStatus = true;
			efi = fsad_access_frame_parsing(ucData, usSize, &af);
			total_id =  af.data_size / APKG_UNSIGNED_CHAR_MAX_NUM;
			if(efi == WCP_OK && gWCPHandler.pFuncCallBack != NULL) {
				for(int i = 0; i < af.data_size; i += APKG_UNSIGNED_CHAR_MAX_NUM) {
					sz = (af.data_size - i) > APKG_UNSIGNED_CHAR_MAX_NUM  ? APKG_UNSIGNED_CHAR_MAX_NUM  : (af.data_size - i);
					gWCPHandler.pFuncCallBack((unsigned char *)send_data_buff, af.code + (cnt * APKG_UNSIGNED_CHAR_MAX_NUM), sz, ADDR_READ);
					buf = fsad_get_data_pack(af.code, cnt++, total_id, sz, send_data_buff, &size);
					memset(send_data_buff, 0, sizeof(send_data_buff));
					u32SendStartTick = xTaskGetTickCount();
					while(1)
					{
						if(gWCPHandler.chrCB[ucWCPtype](index, buf, size) == true) {
							break;
						}			
						// 内存不足超时判断
						if (abs((long long)xTaskGetTickCount() - (long long)u32SendStartTick) > APKG_FREE_SIZE_TIME_OUT)
						{
							ESP_LOGD("apkg_size"," time out");
							bStatus = false;
							break;
						}	
						vTaskDelay(20 / portTICK_PERIOD_MS);
					}
					if(bStatus == false)	
						return;
				}		
			}
		break;
        case ADDR_ERROR:     
           //错误日志
		   #if DEBUG > 0
				ESP_LOGD("cm","this frame is error");
		   #endif
		break;
	} 
	WCP_GIVE(gWCPCBSemaphore);
}

/**
 *                             在中央设备中，访问获取的数据并判断数据          
 * @param    usCode            访问的功能码
 * @param    usSize            访问的数据大小
 * @param    usData            目的地数据
 * @return   eWCPinfo          操作的信息
 **/
static eWCPinfo eWcpGetData(unsigned int usCode, unsigned int usSize, unsigned char *usData)
{
	eWCPinfo eif;
	static   unsigned int usCurrent_id = 0;
    if(gDtfa.data_size == 0x0000 && gDtfa.data == NULL) 
	{
		eif = WCP_NO_DATA;	
	}
	if(gDtfa.code == usCode && usCurrent_id == gDtfa.current_id) 
	{
		int j = 0;
		unsigned char  *pusData = NULL;
		pusData = (unsigned char *)gDtfa.data;
		for(int i = gDtfa.current_id * APKG_UNSIGNED_CHAR_MAX_NUM ; i < gDtfa.current_id * APKG_UNSIGNED_CHAR_MAX_NUM  + gDtfa.data_size ; i++ ) {
			usData[i] = pusData[j++];
		}
		usCurrent_id++;
        g_Flag = true;
	}
	if(gDtfa.current_id * APKG_UNSIGNED_CHAR_MAX_NUM  + gDtfa.data_size == usSize &&  gDtfa.total_pack_id == (usCurrent_id - 1))
	{
		usCurrent_id        = 0;
		gDtfa.code          = 0xFFFF,
        gDtfa.current_id    = 0x0000,
        gDtfa.total_pack_id = 0x0000,
        gDtfa.data_size     = 0x0000,
        gDtfa.data          = NULL,
		eif = WCP_OK;	
	}
	else
	{
		eif = WCP_NO_COMPLETE;
	}
	return eif;
}
