/* Includes ------------------------------------------------------------------*/
#include "djson.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>


static DJSON_Type 	DJSON_GetType(char cTypeString);
static char *DJSON_GetMatch(DJSON *pxDJSON, char *pcFindStart, DJSON_Type eType);

/**
  * @brief
  */
DJSON DJSON_GetObjectItem(DJSON *pxDJSON, char *pcItemName)
{
	DJSON result;
	char cFindName[64];
	
	memset(&result, 0, sizeof(DJSON));
	if (pxDJSON->pcItemPointer != NULL && pcItemName != NULL)
	{
		char *pcNamePointer;

		sprintf(cFindName, "\"%s\":", pcItemName);
		if ((pcNamePointer = strstr(pxDJSON->pcItemPointer, cFindName)) != NULL)
		{
			char *pcTypePointer = NULL;
			char *pcEnd = NULL;
			DJSON_Type eType;
			
			pcTypePointer = pcNamePointer + strlen(cFindName);
			//type
			if (pcTypePointer - pxDJSON->pcItemPointer >= pxDJSON->ulItemLength)
				goto DJSON_fail;
			
			eType = DJSON_GetType(*pcTypePointer);
			//get type
			if (eType == DJSON_Array)
			{
				//Array
				pcEnd = DJSON_GetMatch(pxDJSON, pcTypePointer, DJSON_Array);
				
				if (pcEnd == NULL)
					goto DJSON_fail;
				
				result.pcItemPointer = pcTypePointer;
				result.ulItemLength = pcEnd - result.pcItemPointer + 1;
				result.eItemType = DJSON_Array;
				
				if (result.ulItemLength > 0)
				{
					result.eElementType = DJSON_GetType(*(result.pcItemPointer + 1));
				}
			}
			else if (*pcTypePointer == '{')
			{
				//Object
				pcEnd = DJSON_GetMatch(pxDJSON, pcTypePointer, DJSON_Object);
				
				if (pcEnd == NULL)
					goto DJSON_fail;
				
				result.pcItemPointer = pcTypePointer;
				result.ulItemLength = pcEnd - result.pcItemPointer + 1;
				result.eItemType = DJSON_Object;
			}
			else if (*pcTypePointer == '"')
			{
				//String
				if ((pcEnd = strstr(pcTypePointer + 1, "\"")) == NULL)
					goto DJSON_fail;
				
				if (pcEnd - pxDJSON->pcItemPointer >= pxDJSON->ulItemLength)
					goto DJSON_fail;
				
				result.pcItemPointer = pcTypePointer + 1;
				result.ulItemLength = pcEnd - result.pcItemPointer;
				result.eItemType = DJSON_String;
			}
			else if (*pcTypePointer >= '0' && *pcTypePointer <= '9')
			{		
				//Number
				result.pcItemPointer = pcTypePointer;
				result.ulItemLength = 0;
				result.eItemType = DJSON_Number;
			}
			else
			{
				goto DJSON_fail;
			}
		}
	}

DJSON_fail:
	return result;
}

unsigned long DJSON_GetArraySize(DJSON *pxDJSON)
{
	char *pcCommaPointer = NULL;
	unsigned long ulArraySize = 0;
	
	if (pxDJSON->eItemType == DJSON_Array && 
		pxDJSON->eElementType != DJSON_None && 
		pxDJSON->ulItemLength != 0)
	{
		if (pxDJSON->eElementType == DJSON_Array || pxDJSON->eElementType == DJSON_Object)
		{
			char *pcMatched = NULL;
			char *pcHeadPointer = NULL;
			
			pcHeadPointer = pxDJSON->pcItemPointer + 1;
			while (pcHeadPointer - pxDJSON->pcItemPointer < pxDJSON->ulItemLength)
			{
				pcMatched = DJSON_GetMatch(pxDJSON, pcHeadPointer, pxDJSON->eElementType);
				
				if (pcMatched != NULL)
				{
					ulArraySize++;
					if (pxDJSON->eElementType == DJSON_Array)
						pcHeadPointer = strstr(pcMatched, "[");
					else
						pcHeadPointer = strstr(pcMatched, "{");
				}
				else 
				{
					break;
				}
				
				if (pcHeadPointer == NULL)
					break;
			}
		}
		else if (pxDJSON->eElementType == DJSON_String || pxDJSON->eElementType == DJSON_Number)
		{
			pcCommaPointer = pxDJSON->pcItemPointer;
			while ((pcCommaPointer = strstr(pcCommaPointer, ",")) != NULL)
			{
				if (pcCommaPointer - pxDJSON->pcItemPointer < pxDJSON->ulItemLength)
				{
					ulArraySize++;
					pcCommaPointer += strlen(",");
				}
				else
				{
					break;
				}
			}
		}
	}
	
	return ulArraySize;
}

unsigned long DJSON_GetArrayInt(DJSON *pxDJSON, char *pcbuf)
{
	char *pcCommaPointer = NULL;
	char *pcFindPointer = NULL;
    char *pcStartPointer = NULL;
	char *pcStopPointer = NULL;
	unsigned long ulArraySize = 0;
	char cdata[4] = {0};
	int cnum = 0;
	if (pxDJSON->eItemType == DJSON_Array && pxDJSON->eElementType != DJSON_None && pxDJSON->ulItemLength != 0)
	{
		if (pxDJSON->eElementType == DJSON_Number)
		{
			pcCommaPointer = pxDJSON->pcItemPointer;
			pcStartPointer = strchr(pcCommaPointer, '[');
			pcStopPointer = strchr(pcCommaPointer, ']');

			if(pcStartPointer == NULL || pcStopPointer== NULL)
			{
				return 0;
			}
			pcStartPointer++;
			while(pcStartPointer < pcStopPointer)
			{
				pcFindPointer = strchr(pcStartPointer, ',');
				if(pcFindPointer == NULL)
				{
					pcFindPointer = strchr(pcStartPointer, ']');
					if(pcFindPointer == NULL)
						break;
				}
				if(pcFindPointer > pcStopPointer)
					pcFindPointer = pcStopPointer;
				memset(cdata, 0,sizeof(cdata));
				cnum = 0;
				while(pcStartPointer < pcFindPointer)
				{
					if(*pcStartPointer != ']' && *pcStartPointer != ' ')
					{
						cdata[cnum++] = *pcStartPointer;
						if(cnum >=3)
							return 0;
					}
					pcStartPointer++;
				}
				pcStartPointer = pcFindPointer + 1;
				pcbuf[ulArraySize++] = atoi(cdata);
				if(ulArraySize > 30)
					return 0;
			}
		}		
	}
	return ulArraySize;
}

DJSON DJSON_GetArrayItem(DJSON *pxDJSON, unsigned long ulIndex)
{
	DJSON result;
	char *pcCommaPointer = NULL;
	char *pcHead = NULL;
	char *pcTail = NULL;
	
	memset(&result, 0, sizeof(DJSON));
	if (pxDJSON->eItemType == DJSON_Array && 
		pxDJSON->eElementType != DJSON_None && 
		pxDJSON->ulItemLength != 0)
	{
		if (pxDJSON->eElementType == DJSON_Array || pxDJSON->eElementType == DJSON_Object)
		{
			char *pcMatched = NULL;
			char *pcHeadPointer = NULL;
			
			pcHeadPointer = pxDJSON->pcItemPointer + 1;
			ulIndex += 1;
			while (ulIndex)
			{
				ulIndex--;
				pcHead = pcHeadPointer;
				pcMatched = DJSON_GetMatch(pxDJSON, pcHeadPointer, pxDJSON->eElementType);
				
				if (pcMatched != NULL)
				{
					pcTail = pcMatched;
					if (pxDJSON->eElementType == DJSON_Array)
						pcHeadPointer = strstr(pcMatched, "[");
					else
						pcHeadPointer = strstr(pcMatched, "{");
				}
				else 
				{
					break;
				}
				
				if (pcHeadPointer == NULL)
				{
					break;
				}
			}
		}
		else if (pxDJSON->eElementType == DJSON_String || pxDJSON->eElementType == DJSON_Number)
		{
			char cFindStr[4];
			
			pcCommaPointer = pxDJSON->pcItemPointer;
		
			if (pxDJSON->eElementType == DJSON_String)
			{
				sprintf(cFindStr, "\",");
				pcHead = pcCommaPointer + 1;
				pcTail = strstr(pcHead, "\"");
			}
			else
			{
				sprintf(cFindStr, ",");
				pcHead = pcCommaPointer;
				pcTail = pcHead;
			}
			
			while (ulIndex)
			{
				ulIndex--;
				if ((pcCommaPointer = strstr(pcCommaPointer, cFindStr)) != NULL)
				{
					if (pcCommaPointer - pxDJSON->pcItemPointer < pxDJSON->ulItemLength)
					{
						pcCommaPointer += strlen(cFindStr);
						pcHead = pcCommaPointer;
						if (pxDJSON->eElementType == DJSON_String)
							pcTail = strstr(pcHead, "\"");
						else
							pcTail = pcHead;
					}
					else
					{
						pcHead = NULL;
						pcTail = NULL;
						break;
					}
				}
			}
		}
		
		if (ulIndex == 0 && 
			pcHead != NULL && pcTail != NULL &&
			pcHead - pxDJSON->pcItemPointer < pxDJSON->ulItemLength &&
			pcTail - pxDJSON->pcItemPointer < pxDJSON->ulItemLength)
		{
			result.pcItemPointer = pcHead;
			result.ulItemLength = pcTail - pcHead + 1;
			result.eItemType = pxDJSON->eElementType;
		}
	}
	
	return result;	
}

void DJSON_GetValueString(DJSON *pxDJSON, char *pcInputBuffer, unsigned long ulBufferSize)
{
	if (pxDJSON->eItemType != DJSON_String || pxDJSON->pcItemPointer == NULL)
		return;
	
	if (ulBufferSize - 1 >= pxDJSON->ulItemLength)
	{
		strncpy(pcInputBuffer, pxDJSON->pcItemPointer, pxDJSON->ulItemLength);
		pcInputBuffer[pxDJSON->ulItemLength] = 0;
	}
	else
	{
		memset(pcInputBuffer, 0, ulBufferSize);
	}
}

double DJSON_GetValueDouble(DJSON *pxDJSON)
{
	return atof(pxDJSON->pcItemPointer);
}

int DJSON_GetValueInt(DJSON *pxDJSON)
{
	return atoi(pxDJSON->pcItemPointer);
}

static DJSON_Type DJSON_GetType(char cTypeString)
{
	if (cTypeString == '[')
		return DJSON_Array;
	else if (cTypeString == '{')
		return DJSON_Object;
	else if (cTypeString == '"')
		return DJSON_String;
	else if (cTypeString >= '0' && cTypeString <= '9')
		return DJSON_Number;
	else 
		return DJSON_None;
}

static char *DJSON_GetMatch(DJSON *pxDJSON, char *pcFindStart, DJSON_Type eType)
{
	char *pcPreFix = NULL, *pcPostFix = NULL, *pcPrePointer = NULL, *pcPostPointer = NULL;
	char cPreKey[4], cPostKey[4];
	
	if (eType != DJSON_Array && eType != DJSON_Object)
		return NULL;
	
	if (eType == DJSON_Array)
	{
		sprintf(cPreKey, "[");
		sprintf(cPostKey, "]");
	}
	else 
	{	
		sprintf(cPreKey, "{");
		sprintf(cPostKey, "}");
	}
	
	pcPrePointer = pcFindStart + 1;
	pcPostPointer = pcFindStart + 1;
	while (1)
	{
		if (eType == DJSON_Array)
		{
			pcPreFix = strstr(pcPrePointer, "[");
			pcPostFix = strstr(pcPostPointer, "]");
		}
		else 
		{
			pcPreFix = strstr(pcPrePointer, "{");
			pcPostFix = strstr(pcPostPointer, "}");
		}
		
		if (pcPreFix != NULL)
			pcPrePointer = pcPreFix + 1;
		if (pcPostFix != NULL)
			pcPostPointer = pcPostFix + 1;
		
		if ((pcPreFix == NULL && pcPostFix != NULL) || (pcPreFix > pcPostFix))
			break;
		
		if (pcPostFix == NULL || pcPreFix == NULL)
			break;
	}
	
	if (pcPostFix - pxDJSON->pcItemPointer < pxDJSON->ulItemLength)
		return pcPostFix;
	else
		return NULL;
}

DJSON DJSON_GetArrayIItem(DJSON *pxDJSON, unsigned long ulIndex)
{
	DJSON result;
	unsigned int start = 0;
	unsigned int end = 0;
	unsigned int length = 0;
	unsigned int num = 0;
	
	memset(&result, 0, sizeof(DJSON));
	if (pxDJSON->eItemType == DJSON_Array && 
		pxDJSON->eElementType != DJSON_None && 
		pxDJSON->ulItemLength != 0)
	{
		if (pxDJSON->eElementType == DJSON_Array || pxDJSON->eElementType == DJSON_Object)
		{
			char *pcHeadPointer = NULL;
			pcHeadPointer = pxDJSON->pcItemPointer + 1;
			length = length + 2;
			ulIndex++;
			while(ulIndex)
			{
				start = 0;
				end = 0;
				num = 1;
				ulIndex--;
				start++;
				result.pcItemPointer = pxDJSON->pcItemPointer + length - 1;
				if(*pcHeadPointer == '[')
				{ 
					while(start != end)
					{
						pcHeadPointer++;
						if(*pcHeadPointer == '[')
							start++;
						else if(*pcHeadPointer == ']')
							end++;
						length++;
						num++;
						if(start < end)
							return result;
					}
					// [ 开始
					pcHeadPointer = pcHeadPointer + 2;
					result.eItemType = DJSON_Array;
				}
				else if(*pcHeadPointer == '{')
				{
					while(start != end)
					{
						pcHeadPointer++;
						if(*pcHeadPointer == '{')
							start++;
						else if(*pcHeadPointer == '}')
							end++;
						length++;
						num++;
						if(start < end)
							return result;
					}
					// { 开始
					pcHeadPointer = pcHeadPointer + 2;
					result.eItemType = DJSON_Object;
				}
				length = length + 2;		
		 }
		 result.ulItemLength = num;
	  }
 }
	return result;	
}