/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include "tim.h"

/* Private macros ------------------------------------------------------------*/
#define TMR_MAX_PORT_NUM		   	4

/* Private types -------------------------------------------------------------*/
typedef struct
{
  unsigned long timeus_out;
  esp_timer_handle_t g_timer;
  const esp_timer_create_args_t g_timer_args;
} TMR;

typedef struct
{
	void (*pFuncUpdateHandler)(void);
} TMR_Handler;

/* Private functions ---------------------------------------------------------*/
static void TMR1_IRQHandler(void *para);
static void TMR2_IRQHandler(void *para);
static void TMR3_IRQHandler(void *para);
static void TMR4_IRQHandler(void *para);

/* Private constants ---------------------------------------------------------*/
static TMR g_xTMR[TMR_MAX_PORT_NUM + 1] = {
	[0] = {
      .timeus_out = 0,
      .g_timer_args = {
        .callback = NULL,
        .name = NULL,
      },
    },
  [1] = {
      .timeus_out = 0,
      .g_timer_args = {
        .callback = &TMR1_IRQHandler,
        .name = "tmr1",
      }, 
  },
  [2] = {
      .timeus_out = 0,
      .g_timer_args = {
        .callback = &TMR2_IRQHandler,
        .name = "tmr2",
      }, 
  },
  [3] = {
      .timeus_out = 0,
      .g_timer_args = {
        .callback = &TMR3_IRQHandler,
        .name = "tmr3",
      }, 
  },
  [4] = {
      .timeus_out = 0,
      .g_timer_args = {
          .callback = &TMR4_IRQHandler,
          .name = "tmr4",
        }, 
  }
};

static TMR_Handler g_xTMRHandler[TMR_MAX_PORT_NUM + 1];

/**
  * @brief  TMR TMRer Init
  * @param  ulPort: 1- 4
  * @param  ulPrescaler
  * @param  ulCounter
  * @retval None
  */
void TMRx_TimerInit(unsigned long ulPort, unsigned long ulPrescaler, unsigned long ulCounter)
{
	if (ulPort > TMR_MAX_PORT_NUM)
		return;
  g_xTMR[ulPort].timeus_out = ulCounter;
  esp_timer_create(&g_xTMR[ulPort].g_timer_args, &g_xTMR[ulPort].g_timer);
}

/**
  * @brief  TMR TMRer set priority
  * @param  bUpdate: true / false
  * @retval None
  */
void TMRx_SetPriority(unsigned long ulPort, unsigned long ulPriority)
{
	if (ulPort > TMR_MAX_PORT_NUM)
		return;
}

/**
  * @brief  TMR TMRer Start (enable update it)
  * @param  ulPort: 1-7
  * @retval None
  */
void TMRx_TimerStart(unsigned long ulPort)
{
	if (ulPort > TMR_MAX_PORT_NUM)
		return;
  esp_timer_start_once(g_xTMR[ulPort].g_timer, g_xTMR[ulPort].timeus_out);
}

/**
  * @brief  TMR TMRer end (disable update it)
  * @param  ulPort: 1-7
  * @retval None
  */
void TMRx_TimerEnd(unsigned long ulPort)
{
	if (ulPort > TMR_MAX_PORT_NUM)
		return;
  esp_timer_stop(g_xTMR[ulPort].g_timer);
}

void TMRx_SetCallBack(unsigned long ulPort, void (*pFuncUpdateCallBack)(void))
{
	if (ulPort > TMR_MAX_PORT_NUM)
		return;
	
	g_xTMRHandler[ulPort].pFuncUpdateHandler = pFuncUpdateCallBack;
}

/* Interrupt Handlers ----------------------------------------------------*/
//TMR1
static void TMR1_IRQHandler(void *para)
{
  if (g_xTMRHandler[1].pFuncUpdateHandler != NULL)
    g_xTMRHandler[1].pFuncUpdateHandler();
}

//TMR2
static void TMR2_IRQHandler(void *para)
{
	
  if (g_xTMRHandler[2].pFuncUpdateHandler != NULL)
    g_xTMRHandler[2].pFuncUpdateHandler();
}

//TMR3
static void TMR3_IRQHandler(void * para)
{
  if (g_xTMRHandler[3].pFuncUpdateHandler != NULL)
    g_xTMRHandler[3].pFuncUpdateHandler();
}

//TMR4
static void TMR4_IRQHandler(void *para)
{
  if (g_xTMRHandler[4].pFuncUpdateHandler != NULL)
    g_xTMRHandler[4].pFuncUpdateHandler();
}


