/* Includes ------------------------------------------------------------------*/
#include "mbfunc.h"
#include "mbmaster.h"
#include "WIT.h"
#include "modbus_update.h"

// Slave call back
eMBErrorCode
eMBRegInputCB(UCHAR *pucRegBuffer, USHORT usAddress, USHORT usNRegs)
{
	return MB_ENOREG;
}

eMBErrorCode
eMBRegHoldingCB(UCHAR *pucRegBuffer, USHORT usAddress, USHORT usNRegs, eMBRegisterMode eMode)
{
	unsigned short usValue;

	// FreeModbus usAddress = request address + 1
	usAddress--;
	switch (eMode)
	{
	case MB_REG_READ:
		while (usNRegs)
		{
			/*------------------------------ Serial Number Address ------------------------------*/
			if (usAddress >= WIT_MB_SN_ADDR && usAddress < WIT_MB_SN_ADDR + WIT_MB_SN_COUNT)
			{
				usValue = WIT_MB_GetSerialNumber(usAddress);
			}
			/*------------------------------ Version Address ------------------------------*/
			else if (usAddress == WIT_MB_VERSION_ADDR)
			{
				usValue = WIT_MB_GetVersion();
			}
			/*------------------------------ Module Name Address ------------------------------*/
			else if (usAddress >= WIT_MB_MODULE_NAME_ADDR && usAddress < WIT_MB_MODULE_NAME_ADDR + WIT_MB_MODULE_NAME_COUNT)
			{
				usValue = WIT_MB_GetModuleName(usAddress);
			}
			/*------------------------------ MQTT Broker Address ------------------------------*/
			else if (usAddress >= WIT_MB_MQTT_BROKER_ADDR && usAddress < WIT_MB_MQTT_BROKER_ADDR + WIT_MB_MQTT_BROKER_COUNT)
			{
				usValue = WIT_MB_GetMQTTBroker(usAddress);
			}
			/*------------------------------ MQTT Port Address ------------------------------*/
			else if (usAddress == WIT_MB_MQTT_PORT_ADDR)
			{
				usValue = WIT_MB_GetMQTTPort();
			}
			/*------------------------------ MQTT User Address ------------------------------*/
			else if (usAddress >= WIT_MB_MQTT_USER_ADDR && usAddress < WIT_MB_MQTT_USER_ADDR + WIT_MB_MQTT_USER_COUNT)
			{
				usValue = WIT_MB_GetMQTTUser(usAddress);
			}
			/*------------------------------ MQTT Password Address ------------------------------*/
			else if (usAddress >= WIT_MB_MQTT_PASSWORD_ADDR && usAddress < WIT_MB_MQTT_PASSWORD_ADDR + WIT_MB_MQTT_PASSWORD_COUNT)
			{
				usValue = WIT_MB_GetMQTTPassword(usAddress);
			}
			/*------------------------------ Alarm Address ------------------------------*/
			else if (usAddress >= WIT_MB_ALARM_ADDR && usAddress < WIT_MB_ALARM_ADDR + WIT_GetGWAndSensorFeatureCount())
			{
				usValue = WIT_MB_GetAlarm(usAddress);
			}
			/*------------------------------ Sensor Exist Status Address ------------------------------*/
			else if (usAddress == WIT_MB_SENSOR_EXIST_STATUS_ADDR)
			{
				usValue = WIT_MB_GetSensorExistStatus();
			}
			/*------------------------------ Sensor Temperature Address ------------------------------*/
			else if (usAddress == WIT_MB_SENSOR_TEMP_ADDR)
			{
				usValue = WIT_MB_GetSensorTemperture();
			}
			/*------------------------------ Sensor Feature Address ------------------------------*/
			else if (usAddress >= WIT_MB_SENSOR_FEATURE_ADDR && usAddress < WIT_MB_SENSOR_FEATURE_ADDR + WIT_MB_SENSOR_FEATURE_COUNT)
			{
				usValue = WIT_MB_GetSensorFeature(usAddress);
			}
			/*------------------------------ Get Sensor ID ------------------------------*/
			else if (usAddress == WIT_MB_SENSOR_ID_ADDR)
			{
				usValue = WIT_MB_GetSensorID();
			}
			/*------------------------------ Get Modbus ID ------------------------------*/
			else if (usAddress == WIT_MB_ID_ADDRESS)
			{
				usValue = WIT_MB_GetID();
			}
			/*------------------------------ Get Modbus BaudRate ------------------------------*/
			else if (usAddress >= WIT_MB_BAUD_RATE_ADDRESS && usAddress < WIT_MB_BAUD_RATE_ADDRESS + WIT_MB_BAUD_RATE_COUNT)
			{
				usValue = WIT_MB_GetBaudRate(usAddress);
			}
			/*------------------------------ Get Modbus Parity ------------------------------*/
			else if (usAddress == WIT_MB_PARITY_ADDRESS)
			{
				usValue = WIT_MB_GetParity();
			}
			/*------------------------------ Update Index Address ------------------------------*/
			// else if (usAddress == MBUPDATE_INDICATION_ADDRESS)
			// {
			// 	usValue = MBUpdate_GetIndex();
			// }
      else if(usAddress >= WIT_MB_SENSOR_NUMBER_ONE && usAddress < WIT_MB_SENSOR_NUMBER_ONE + WIT_MB_SENSOR_NUMBER_TOTAL_SIZE)
			{
				usValue = WIT_MB_GetNumberSensorFeature(usAddress);
			}
			else
			{
				usValue = 0;
			}

			*pucRegBuffer++ = (UCHAR)(usValue >> 8);
			*pucRegBuffer++ = (UCHAR)usValue;
			usNRegs--;
			usAddress++;
		}
		break;
	case MB_REG_WRITE:
		while (usNRegs)
		{
			usValue = (USHORT)(*pucRegBuffer++) << 8;
			usValue |= *pucRegBuffer++;
			/*------------------------------ SENSOR Channel Count Address ------------------------------*/
			if (usAddress == WIT_MB_SENSOR_CHANNEL_COUNT_ADDR)
			{
				WIT_MB_SetChannelCount(usValue);
			}
			/*------------------------------ MQTT Broker Address ------------------------------*/
			else if (usAddress >= WIT_MB_MQTT_BROKER_ADDR && usAddress < WIT_MB_MQTT_BROKER_ADDR + WIT_MB_MQTT_BROKER_COUNT)
			{
				WIT_MB_SetMQTTBroker(usAddress, usValue);
			}
			/*------------------------------ MQTT Port Address ------------------------------*/
			else if (usAddress == WIT_MB_MQTT_PORT_ADDR)
			{
				WIT_MB_SetMQTTPort(usValue);
			}
			/*------------------------------ MQTT User Address ------------------------------*/
			else if (usAddress >= WIT_MB_MQTT_USER_ADDR && usAddress < WIT_MB_MQTT_USER_ADDR + WIT_MB_MQTT_USER_COUNT)
			{
				WIT_MB_SetMQTTUser(usAddress, usValue);
			}
			/*------------------------------ MQTT Password Address ------------------------------*/
			else if (usAddress >= WIT_MB_MQTT_PASSWORD_ADDR && usAddress < WIT_MB_MQTT_PASSWORD_ADDR + WIT_MB_MQTT_PASSWORD_COUNT)
			{
				WIT_MB_SetMQTTPassword(usAddress, usValue);
			}
			/*------------------------------ Set Sensor ID ------------------------------*/
			else if (usAddress == WIT_MB_SENSOR_ID_ADDR)
			{
				WIT_MB_SetSensorID(usValue);
			}
			/*------------------------------ Set Modbus ID ------------------------------*/
			else if (usAddress == WIT_MB_ID_ADDRESS)
			{
				WIT_MB_SetID(usValue);
			}
			/*------------------------------ Set Modbus BaudRate ------------------------------*/
			else if (usAddress >= WIT_MB_BAUD_RATE_ADDRESS && usAddress < WIT_MB_BAUD_RATE_ADDRESS + WIT_MB_BAUD_RATE_COUNT)
			{
				WIT_MB_SetBaudRate(usAddress, usValue);
			}
			/*------------------------------ Set Modbus Parity ------------------------------*/
			else if (usAddress == WIT_MB_PARITY_ADDRESS)
			{
				WIT_MB_SetParity(usValue);
			}
			/*--------------------------------------------- Update Address---------------------------------------------*/
			// else if (usAddress >= MBUPDATE_RENEW_ADDRESS && usAddress < MBUPDATE_RENEW_ADDRESS + MBUPDATE_RENEW_PER_SIZE)
			// {
			// 	MBUpdate_WriteToBufferAndFlash(usAddress, usValue);
			// }
			// else if (usAddress == MBUPDATE_INDICATION_ADDRESS)
			// {
			// 	MBUpdate_SetIndex(usValue);
			// }
			// else if (usAddress >= MBUPDATE_CONFIRM_ADDRESS && usAddress < MBUPDATE_CONFIRM_ADDRESS + MBUPDATE_CONFIRM_SIZE)
			// {
			// 	MBUpdate_SetPassword(usAddress, usValue);
			// }
			usNRegs--;
			usAddress++;
		}
		break;
	}

	return MB_ENOERR;
}

eMBErrorCode
eMBRegCoilsCB(UCHAR *pucRegBuffer, USHORT usAddress, USHORT usNCoils,
			  eMBRegisterMode eMode)
{
	return MB_ENOREG;
}

eMBErrorCode
eMBRegDiscreteCB(UCHAR *pucRegBuffer, USHORT usAddress, USHORT usNDiscrete)
{
	return MB_ENOREG;
}

// Master call back
void vMBReadInputRegisterCB(UCHAR *pucData, USHORT usDataLen, UCHAR ucModbusPort)
{
	vMBSetResponseData(pucData, usDataLen, ucModbusPort);
}

void vMBReadHoldingRegisterCB(UCHAR *pucData, USHORT usDataLen, UCHAR ucModbusPort)
{
	vMBSetResponseData(pucData, usDataLen, ucModbusPort);
}

void vMBWriteHoldingRegisterCB(UCHAR *pucData, USHORT usDataLen, UCHAR ucModbusPort)
{
	vMBSetResponseData(pucData, usDataLen, ucModbusPort);
}

void vMBWriteMultipleHoldingRegisterCB(UCHAR *pucData, USHORT usDataLen, UCHAR ucModbusPort)
{
	vMBWriteMultipleRegisterResponseData(pucData, usDataLen, ucModbusPort);
}

void vMBReadCoilsCB(UCHAR *pucData, USHORT usDataLen, UCHAR ucModbusPort)
{
	vMBSetResponseData(pucData, usDataLen, ucModbusPort);
}

void vMBWriteCoilCB(UCHAR *pucData, USHORT usDataLen, UCHAR ucModbusPort)
{
	vMBWriteSingResponseResponseData(pucData, usDataLen, ucModbusPort);
}

void vMBWriteMultipleCoilsCB(UCHAR *pucData, USHORT usDataLen, UCHAR ucModbusPort)
{
	vMBWriteMultipleRegisterResponseData(pucData, usDataLen, ucModbusPort);
}

void vMBReadDiscreteInputsCB(UCHAR *pucData, USHORT usDataLen, UCHAR ucModbusPort)
{
}

void vMBReadWriteMultipleHoldingRegisterCB(UCHAR *pucData, USHORT usDataLen, UCHAR ucModbusPort)
{
}
