/* ----------------------- System includes ----------------------------------*/
#include "stdlib.h"
#include "string.h"

/* ----------------------- Platform includes --------------------------------*/
#include "port.h"

/* ----------------------- Modbus includes ----------------------------------*/
#include "mb.h"
#include "mbframe.h"
#include "mbproto.h"
#include "mbconfig.h"

#define MB_PDU_FUNC_OFF				0
#define MB_PDU_ID_LEN_OFF			1
#define MB_PDU_SIZE_TYPE_A			10
#define MB_PDU_SIZE_TYPE_B			12
#define MB_PDU_FILE_DATA_OFF_A 		4
#define MB_PDU_FILE_DATA_OFF_B		6
#define MB_FILE_DATA_NUMBER_OFF		0
#define MB_FILE_DATA_ADDRESS_OFF	2
#define MB_FILE_DATA_COUNT_OFF		4
#define MB_PDU_READ_MAX_SIZE		200

/* ----------------------- Static functions ---------------------------------*/
eMBException    prveMBError2Exception( eMBErrorCode eErrorCode );

#if MB_FUNC_READ_FILE_ENABLED > 0
eMBException    eMBFuncReadFile( UCHAR * pucFrame, USHORT * usLen )
{
	USHORT			usFileNumber;
	USHORT         	usRegAddress;
    USHORT         	usRegCount;
    UCHAR          	*pucFrameCur;
	UCHAR			*pucFilePointer;

    eMBException    eStatus = MB_EX_NONE;
    eMBErrorCode    eRegStatus;
	
	if( *usLen == MB_PDU_SIZE_TYPE_A ||
		*usLen == MB_PDU_SIZE_TYPE_B)
	{
		if (*usLen == MB_PDU_SIZE_TYPE_A)
			pucFilePointer = &pucFrame[MB_PDU_FILE_DATA_OFF_A];
		else
			pucFilePointer = &pucFrame[MB_PDU_FILE_DATA_OFF_B];
		
		usFileNumber = pucFilePointer[MB_FILE_DATA_NUMBER_OFF] | ((USHORT)(pucFilePointer[MB_FILE_DATA_NUMBER_OFF + 1]) << 8);
		usRegAddress = pucFilePointer[MB_FILE_DATA_ADDRESS_OFF] | ((USHORT)(pucFilePointer[MB_FILE_DATA_ADDRESS_OFF + 1]) << 8);
		usRegCount = pucFilePointer[MB_FILE_DATA_COUNT_OFF] | ((USHORT)(pucFilePointer[MB_FILE_DATA_COUNT_OFF + 1]) << 8);
		
		if (usRegCount >= 1 && usRegCount <= MB_PDU_READ_MAX_SIZE)
		{
			pucFrameCur = pucFrame;
			pucFrameCur[MB_PDU_FUNC_OFF] = MB_FUNC_READ_FILE_RESPONSE;
			pucFrameCur = &pucFilePointer[MB_FILE_DATA_COUNT_OFF + 2];
			eRegStatus = eMBRegFileCB(pucFrameCur, usFileNumber, usRegAddress, usRegCount, MB_REG_READ);
			
			if (eRegStatus != MB_ENOERR)
			{
				eStatus = prveMBError2Exception( eRegStatus );
			}
			else
			{
				*usLen += usRegCount;
			}
		}
		else
		{
			eStatus = MB_EX_ILLEGAL_DATA_VALUE;
		}
	}
	else
    {
        /* Can't be a valid request because the length is incorrect. */
        eStatus = MB_EX_ILLEGAL_DATA_VALUE;
    }
    return eStatus;
}
#endif

#if MB_FUNC_WRITE_FILE_ENABLED > 0
eMBException    eMBFuncWriteFile( UCHAR * pucFrame, USHORT * usLen )
{
	USHORT			usFileNumber;
	USHORT         	usRegAddress;
    USHORT         	usRegCount;
    UCHAR          	*pucFrameCur;
	UCHAR			*pucFilePointer;

    eMBException    eStatus = MB_EX_NONE;
    eMBErrorCode    eRegStatus;
	
	if( *usLen >= MB_PDU_SIZE_TYPE_A)
	{
		if (pucFrame[MB_PDU_ID_LEN_OFF] == 0x01)
			pucFilePointer = &pucFrame[MB_PDU_FILE_DATA_OFF_A];
		else
			pucFilePointer = &pucFrame[MB_PDU_FILE_DATA_OFF_B];
		
		usFileNumber = pucFilePointer[MB_FILE_DATA_NUMBER_OFF] | ((USHORT)(pucFilePointer[MB_FILE_DATA_NUMBER_OFF + 1]) << 8);
		usRegAddress = pucFilePointer[MB_FILE_DATA_ADDRESS_OFF] | ((USHORT)(pucFilePointer[MB_FILE_DATA_ADDRESS_OFF + 1]) << 8);
		usRegCount = pucFilePointer[MB_FILE_DATA_COUNT_OFF] | ((USHORT)(pucFilePointer[MB_FILE_DATA_COUNT_OFF + 1]) << 8);
		
		if (usRegCount >= 1 && usRegCount <= MB_PDU_READ_MAX_SIZE)
		{
			pucFrameCur = pucFrame;
			pucFrameCur[MB_PDU_FUNC_OFF] = MB_FUNC_WRITE_FILE_RESPONSE;
			pucFrameCur = &pucFilePointer[MB_FILE_DATA_COUNT_OFF + 2];
			eRegStatus = eMBRegFileCB(pucFrameCur, usFileNumber, usRegAddress, usRegCount, MB_REG_WRITE);
			
			if (eRegStatus != MB_ENOERR)
			{
				eStatus = prveMBError2Exception( eRegStatus );
			}
			else
			{
				*usLen = 10;
				*pucFrameCur = MB_ENOERR;
				*usLen += 1;
			}
		}
		else
		{
			eStatus = MB_EX_ILLEGAL_DATA_VALUE;
		}
	}
	else
    {
        /* Can't be a valid request because the length is incorrect. */
        eStatus = MB_EX_ILLEGAL_DATA_VALUE;
    }
    return eStatus;
}
#endif
