/* Includes ------------------------------------------------------------------*/
#include <string.h>
#include "mbmaster.h"
#include "mbrtu.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"

/* Private macros ------------------------------------------------------------*/
#define MB_MASTER_MAX_BUFFER_SIZE		256
//#define MB_MASTER_TIMEOUT				2000
#define MB_MASTER_PER_WAIT_TIME			1
#define MB_MASTER_PDU_INDEX				1

/* Private functions ---------------------------------------------------------*/
static USHORT		usMBPackage(const UCHAR ucModbusPort,const UCHAR ucSlaveAddress, const UCHAR ucCommand, const USHORT usStartAddress, const USHORT usNumberorData, const UCHAR * pucDataArray);
static eMBErrorCode	eMBWaitForResponse(unsigned long ulTimeout,UCHAR ucModbusPort);
static void 		vMBPortMutexLock(unsigned char ucModbusPort);
static void 		vMBPortMutexUnlock(unsigned char ucModbusPort);
static SemaphoreHandle_t g_xMBMasterCommandHandle[MODBUS_MAX_PORT_NUM];
/* Private variables ---------------------------------------------------------*/
static UCHAR 			g_ucCurrentReadSlaveID[MODBUS_MAX_PORT_NUM];

static UCHAR			g_ucMasterCommandBuffer[MODBUS_MAX_PORT_NUM][MB_MASTER_MAX_BUFFER_SIZE];
static UCHAR			g_ucMasterResponseBuffer[MODBUS_MAX_PORT_NUM][MB_MASTER_MAX_BUFFER_SIZE];
static UCHAR			g_ucMasterResponseLen;
static USHORT			g_usMasterTimeout = 500;

/**
  * @brief  get current read slave address
  * @retval slave modbus id
  */
UCHAR 
ucMBGetCurrentReadSlaveID(UCHAR ucModbusPort)
{
	return g_ucCurrentReadSlaveID[MB_PORT_TO_INDEX(ucModbusPort)];
}

void 		 vMBSetTimeout(USHORT usTimeout)
{
	g_usMasterTimeout = usTimeout;
}

void
vMBSetResponseData(UCHAR *pucData, UCHAR ucDataLen,UCHAR ucModbusPort)
{
	g_ucMasterResponseLen = ucDataLen;
	memcpy(g_ucMasterResponseBuffer[MB_PORT_TO_INDEX(ucModbusPort)], pucData, ucDataLen);
}

void
vMBWriteMultipleRegisterResponseData(UCHAR *pucData, UCHAR ucDataLen,UCHAR ucModbusPort)
{
	g_ucMasterResponseLen = pucData[2] * 2;
}
void
vMBWriteSingResponseResponseData(UCHAR *pucData, UCHAR ucDataLen,UCHAR ucModbusPort)
{
    g_ucMasterResponseLen = 2;
    memcpy(g_ucMasterResponseBuffer[MB_PORT_TO_INDEX(ucModbusPort)], pucData - 1, 2);
}
    
/**
  * @brief  read coil
  * @retval eMBErrorCode
  */
eMBErrorCode eMBReadCoil(const UCHAR ucModbusPort,const UCHAR ucSlaveAddress, const USHORT usStartAddress, const USHORT usCoilNumber, UCHAR *pusReceiveDataBuffer)
{
	eMBErrorCode	eStatus;
	UCHAR			ucPackagedPDULen;
    vMBPortMutexLock(ucModbusPort);
	ucPackagedPDULen = usMBPackage(ucModbusPort,ucSlaveAddress, MB_FUNC_READ_COILS, usStartAddress, usCoilNumber, NULL);
	//PDU Len
	eMBRTUSend(ucSlaveAddress, &g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][MB_MASTER_PDU_INDEX], ucPackagedPDULen,ucModbusPort);
	if ((eStatus = eMBWaitForResponse(g_usMasterTimeout, ucModbusPort)) == MB_EMASTERNOERR)
	{
		//Check number of data
		if (g_ucMasterResponseLen == (usCoilNumber - 1) / 8 + 1)
		{
			if (pusReceiveDataBuffer != NULL)
			{
				int i;
				for (i = 0;i < (usCoilNumber - 1) / 8 + 1;i++)
				{
					pusReceiveDataBuffer[i] = g_ucMasterResponseBuffer[MB_PORT_TO_INDEX(ucModbusPort)][i];
				}
			}
		}
		else
		{
			eStatus = MB_EMASTERERR;
		}
	}
	vMBPortSerialEnable(FALSE, FALSE,ucModbusPort);
	vTaskDelay(1);
    vMBPortMutexUnlock(ucModbusPort);
	return eStatus;
}

/**
  * @brief  read holding register
  * @retval eMBErrorCode
  */
eMBErrorCode 
eMBReadHoldingRegister(const UCHAR ucModbusPort,const UCHAR ucSlaveAddress, const USHORT usStartAddress, const USHORT usRegisterNumber, USHORT *pusReceiveDataBuffer)
{
	eMBErrorCode	eStatus;
	UCHAR			ucPackagedPDULen;
    vMBPortMutexLock(ucModbusPort);
	ucPackagedPDULen = usMBPackage(ucModbusPort,ucSlaveAddress, MB_FUNC_READ_HOLDING_REGISTER, usStartAddress, usRegisterNumber, NULL);
	//PDU Len
	eMBRTUSend(ucSlaveAddress, &g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][MB_MASTER_PDU_INDEX], ucPackagedPDULen,ucModbusPort);
	if ((eStatus = eMBWaitForResponse(g_usMasterTimeout, ucModbusPort)) == MB_EMASTERNOERR)
	{
		//Check number of data
		if (g_ucMasterResponseLen == usRegisterNumber * 2)
		{
			if (pusReceiveDataBuffer != NULL)
			{
				int i;
				for (i = 0;i < usRegisterNumber;i++)
				{
					pusReceiveDataBuffer[i] = (USHORT)(g_ucMasterResponseBuffer[MB_PORT_TO_INDEX(ucModbusPort)][i * 2] << 8) | g_ucMasterResponseBuffer[MB_PORT_TO_INDEX(ucModbusPort)][i * 2 + 1];
				}
			}
		}
		else
		{
			eStatus = MB_EMASTERERR;
		}
	}
	vMBPortSerialEnable(FALSE, FALSE,ucModbusPort);
	vTaskDelay(1);
    vMBPortMutexUnlock(ucModbusPort);
	return eStatus;
}

/**
  * @brief  read input register
  * @retval eMBErrorCode
  */
eMBErrorCode 
eMBWriteMultiHoldingRegister(const UCHAR ucModbusPort,const UCHAR ucSlaveAddress, const USHORT usStartAddress, const USHORT usRegisterNumber, USHORT *pusDataBuffer)
{
	eMBErrorCode	eStatus;
	UCHAR			ucPackagedPDULen;
	vMBPortMutexLock(ucModbusPort);
	ucPackagedPDULen = usMBPackage(ucModbusPort,ucSlaveAddress, MB_FUNC_WRITE_MULTIPLE_REGISTERS, usStartAddress, usRegisterNumber, (UCHAR *)pusDataBuffer);
	//PDU Len
	eMBRTUSend(ucSlaveAddress, &g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][MB_MASTER_PDU_INDEX], ucPackagedPDULen,ucModbusPort);
	if ((eStatus = eMBWaitForResponse(g_usMasterTimeout, ucModbusPort)) == MB_EMASTERNOERR)
	{
	}
	vMBPortSerialEnable(FALSE, FALSE,ucModbusPort);
	vMBPortMutexUnlock(ucModbusPort);
	return eStatus;
}

/**
  * @brief  read holding register
  * @retval eMBErrorCode
  */
eMBErrorCode 
eMBWriteHoldingRegister(const UCHAR ucModbusPort,const UCHAR ucSlaveAddress, const USHORT usStartAddress, const USHORT usRegisterData)
{
	eMBErrorCode	eStatus;
	UCHAR			ucPackagedPDULen;
	vMBPortMutexLock(ucModbusPort);
	ucPackagedPDULen = usMBPackage(ucModbusPort,ucSlaveAddress, MB_FUNC_WRITE_REGISTER, usStartAddress, usRegisterData, NULL);
	//PDU Len
	eMBRTUSend(ucSlaveAddress, &g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][MB_MASTER_PDU_INDEX], ucPackagedPDULen,ucModbusPort);
	if ((eStatus = eMBWaitForResponse(g_usMasterTimeout, ucModbusPort)) == MB_EMASTERNOERR)
	{
		
	}
	vMBPortSerialEnable(FALSE, FALSE,ucModbusPort);
	vMBPortMutexUnlock(ucModbusPort);
	return eStatus;
}


/**
  * @brief  Write Multiple coil
  * @retval eMBErrorCode
  */
eMBErrorCode 
eMBWriteMultipleCoil(const UCHAR ucModbusPort,const UCHAR ucSlaveAddress, const USHORT usStartAddress, const USHORT usRegisterNumber, const UCHAR *pusSendDataBuffer)
{
	eMBErrorCode	eStatus;
	UCHAR			ucPackagedPDULen;
    vMBPortMutexLock(ucModbusPort);
	ucPackagedPDULen = usMBPackage(ucModbusPort,ucSlaveAddress, MB_FUNC_WRITE_MULTIPLE_COILS, usStartAddress, usRegisterNumber, pusSendDataBuffer);
	//PDU Len
	eMBRTUSend(ucSlaveAddress, &g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][MB_MASTER_PDU_INDEX], ucPackagedPDULen,ucModbusPort);
	if ((eStatus = eMBWaitForResponse(g_usMasterTimeout, ucModbusPort)) == MB_EMASTERNOERR)
	{
		//Check number of data
		if (g_ucMasterResponseLen == usRegisterNumber * 2)
		{
		}
		else
		{
			eStatus = MB_EMASTERERR;
		}
	}
	vMBPortSerialEnable(FALSE, FALSE,ucModbusPort);
	vTaskDelay(1);
    vMBPortMutexUnlock(ucModbusPort);
	return eStatus;
}
/**
  * @brief  command package
  * @retval eMBErrorCode
  */
static USHORT 
usMBPackage(const UCHAR ucModbusPort,const UCHAR ucSlaveAddress, const UCHAR ucCommand, const USHORT usStartAddress,const USHORT usNumberorData, const UCHAR * pucDataArray)
{
	int i = 0;
	USHORT usLen = 0;
	
	g_ucCurrentReadSlaveID[MB_PORT_TO_INDEX(ucModbusPort)] = ucSlaveAddress;
	switch (ucCommand)
	{
	case MB_FUNC_READ_COILS:
	case MB_FUNC_READ_HOLDING_REGISTER:
	case MB_FUNC_READ_INPUT_REGISTER:
	case MB_FUNC_WRITE_REGISTER:
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = ucSlaveAddress;
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = ucCommand;
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)(usStartAddress >> 8);
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)usStartAddress;
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)(usNumberorData >> 8);
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)usNumberorData;
		break;
    case MB_FUNC_WRITE_SINGLE_COIL:	 
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = ucSlaveAddress;
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = ucCommand;
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)(usStartAddress >> 8);
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)usStartAddress;
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] =  pucDataArray[i];
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] =  pucDataArray[i+1];
        break;    
	case MB_FUNC_WRITE_MULTIPLE_COILS:
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = ucSlaveAddress;
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = ucCommand;
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)(usStartAddress >> 8);
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)(usStartAddress);
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)(usNumberorData >> 8);
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)usNumberorData;
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)((usNumberorData - 1) / 8 + 1);
		for (i = 0; i < (usNumberorData - 1) / 8 + 1;i++)
		{
			g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = pucDataArray[i];
		}
		break;
	case MB_FUNC_WRITE_MULTIPLE_REGISTERS:
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = ucSlaveAddress;
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = ucCommand;
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)(usStartAddress >> 8);
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)usStartAddress;
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)(usNumberorData >> 8);
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)usNumberorData;
		g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = (UCHAR)(usNumberorData * 2);
		for (i = 0; i < usNumberorData;i++)
		{
			g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = pucDataArray[i * 2 + 1];
			g_ucMasterCommandBuffer[MB_PORT_TO_INDEX(ucModbusPort)][usLen++] = pucDataArray[i * 2];
		}
		break;
	default:
		break;
	}
	
	return usLen - 1;
}

static 
eMBErrorCode eMBWaitForResponse(unsigned long ulTimeout,UCHAR ucModbusPort)
{
	eMBErrorCode eStatus;
	unsigned long ulTimes = 0;
	do
	{
		eStatus = eMBPoll(ucModbusPort);
		
		if (ucMBGetCurrentReadSlaveID(ucModbusPort) == MB_ADDRESS_BROADCAST && eStatus == MB_EMASTERSENT)
		{
			eStatus = MB_EMASTERNOERR;
			break;
		}
		
		if (eStatus == MB_EMASTERNOERR || eStatus == MB_EMASTERERR)
			break;
		if (g_ucCurrentReadSlaveID[MB_PORT_TO_INDEX(ucModbusPort)] == MB_ADDRESS_BROADCAST && eStatus == MB_EMASTERSENT)
			break;
		ulTimes++;
		if (ulTimes * MB_MASTER_PER_WAIT_TIME >= ulTimeout)
		{
			eStatus = MB_ETIMEDOUT;
			break;
		}
		vTaskDelay(MB_MASTER_PER_WAIT_TIME);
	} while (TRUE);
	
	return eStatus;
}

void 
vMBPortMutexLock(unsigned char ucModbusPort)
{
	if (g_xMBMasterCommandHandle[MB_PORT_TO_INDEX(ucModbusPort)] == NULL)
		g_xMBMasterCommandHandle[MB_PORT_TO_INDEX(ucModbusPort)] = xSemaphoreCreateMutex();//xSemaphoreCreateBinary();
	else
		xSemaphoreTake(g_xMBMasterCommandHandle[MB_PORT_TO_INDEX(ucModbusPort)], portMAX_DELAY);
}

void 
vMBPortMutexUnlock(unsigned char ucModbusPort)
{
	if (g_xMBMasterCommandHandle[MB_PORT_TO_INDEX(ucModbusPort)] == NULL)
		g_xMBMasterCommandHandle[MB_PORT_TO_INDEX(ucModbusPort)] = xSemaphoreCreateMutex();//xSemaphoreCreateBinary();
	else
		xSemaphoreGive(g_xMBMasterCommandHandle[MB_PORT_TO_INDEX(ucModbusPort)]);
}


