/*
 * FreeModbus Libary: BARE Port
 * Copyright (C) 2006 Christian Walter <wolti@sil.at>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * File: $Id: porttimer.c,v 1.1 2006/08/22 21:35:13 wolti Exp $
 */

/* ----------------------- Platform includes --------------------------------*/
#include "port.h"

/* ----------------------- Modbus includes ----------------------------------*/
#include "mb.h"
#include "mbport.h"
#include "tim.h"

/* ----------------------- User includes ------------------------------------*/

/* ----------------------- Private macros -----------------------------------*/
#define MB_TIM_PORT			2
#define MB_TIM_PRIORITY		1
#define MB_FREQUENCY		20000			//20KHz

uint32_t SystemCoreClock = 80000000;
/* ----------------------- static functions ---------------------------------*/
static void prvvTIMERE1piredISR( void );
static void prvvTIMERE2piredISR( void );
static void prvvTIMERE3piredISR( void );
static void prvvTIMERE4piredISR( void );
/* ----------------------- Start implementation -----------------------------*/
static inline unsigned char MB_PORT_TO_TIMEINDEX(unsigned char ucUartPort)
{
    switch(ucUartPort)
	{
		case 1:
			return 2;
		case 2:
			return 3;
		case 3:
			return 4;		
		default:
			return 1;
    }

}
static inline unsigned char MB_TIME_TO_PORTINDEX(unsigned char ucUartPort)
{
    switch(ucUartPort)
	{
		case 2:
			return 1;
		case 3:
			return 2;
		case 4:
			return 3;		
		default:
			return 1;
    }

}
BOOL
xMBPortTimersInit( USHORT usTim1Timerout50us, UCHAR ucModbusPort )
{
	unsigned long ulPrescaler = (SystemCoreClock / MB_FREQUENCY) - 1;
    ucModbusPort = MB_PORT_TO_TIMEINDEX(ucModbusPort);
	TMRx_TimerInit(ucModbusPort, ulPrescaler, usTim1Timerout50us);
	TMRx_SetPriority(ucModbusPort, MB_TIM_PRIORITY + ucModbusPort);
    switch(ucModbusPort)
    {
        case 1:
             TMRx_SetCallBack(1, prvvTIMERE1piredISR);
             break;
        case 2:
             TMRx_SetCallBack(2, prvvTIMERE2piredISR);
             break;       
        case 3:
             TMRx_SetCallBack(3, prvvTIMERE3piredISR);
             break;            
        case 4:
             TMRx_SetCallBack(4, prvvTIMERE4piredISR);
        default:
             break;
    }
    return TRUE;
}


inline void
vMBPortTimersEnable( UCHAR ucModbusPort )
{
    /* Enable the timer with the timeout passed to xMBPortTimersInit( ) */
	TMRx_TimerStart(MB_PORT_TO_TIMEINDEX(ucModbusPort));
}

inline void
vMBPortTimersDisable( UCHAR ucModbusPort )
{
    /* Disable any pending timers. */
	TMRx_TimerEnd(MB_PORT_TO_TIMEINDEX(ucModbusPort));
}

/* Create an ISR which is called whenever the timer has expired. This function
 * must then call pxMBPortCBTimerExpired( ) to notify the protocol stack that
 * the timer has expired.
 */
static void prvvTIMERE1piredISR( void)
{
    ( void )pxMBPortCBTimerExpired(MB_TIME_TO_PORTINDEX(1));
}

static void prvvTIMERE2piredISR(void)
{
    ( void )pxMBPortCBTimerExpired(MB_TIME_TO_PORTINDEX(2));
}
static void prvvTIMERE3piredISR(void)
{
    ( void )pxMBPortCBTimerExpired(MB_TIME_TO_PORTINDEX(3));
}

static void prvvTIMERE4piredISR(void)
{
    ( void )pxMBPortCBTimerExpired(MB_TIME_TO_PORTINDEX(4));
}
