/* Includes ------------------------------------------------------------------*/
#include "eth.h"
#include "w5100s_conf.h"
#include "mqtts.h"
#include "w5100S.h"
#include "dhcp.h"
#include "string.h"
#include  "loopback.h"

extern uint8_t ethBuf[ETH_MAX_BUF_SIZE];
extern wiz_NetInfo gWIZNETINFO;
#define ETHERNET_RECONNECT_TASK_STK	(configMINIMAL_STACK_SIZE * 4)

unsigned long g_ulMaxSocketID = 4;

static void ETHERNET_ReconnectTask(void *pvParameter);

ETHERNET_Socket	g_xETHSocket[ETH_MAX_SOCKET_COUNT];

void (*pFuncMQTTStatusChangedHandler)(unsigned long, bool);
bool (*pFuncMQTTReconnectHandler)(unsigned long);

void ETHERNET_SetMQTTStatusChangedCallBack(void (*pFuncMQTTStatusChangedCallBack)(unsigned long, bool))
{
    //printf("addr init : %lx\r\n", (unsigned long)(*pFuncMQTTStatusChangedCallBack));
	pFuncMQTTStatusChangedHandler = pFuncMQTTStatusChangedCallBack;
    //printf("addr init : %p\r\n", pFuncMQTTStatusChangedHandler);
}

void ETHERNET_SetMQTTReconnectCallBack(bool (*pFuncMQTTReconnectCallBack)(unsigned long))
{
	pFuncMQTTReconnectHandler = pFuncMQTTReconnectCallBack;
}

long ETHERNET_CreateSocket(char *pcMode, char *pcHost, char *pcPort)
{
    unsigned long ulIndex = 0;
	
	if (pcMode == NULL || pcHost == NULL || pcPort == NULL)
		return -1;
	for (ulIndex = 0;ulIndex < ETH_MAX_SOCKET_COUNT;ulIndex++)
	{
		if (!g_xETHSocket[ulIndex].bUsed)
		{
			break;
		}
	}
	if (ulIndex == ETH_MAX_SOCKET_COUNT)
	{
		return -1;
	}
	else
	{
		strcpy(g_xETHSocket[ulIndex].cMode, pcMode);
		strcpy(g_xETHSocket[ulIndex].cDestinationHost, pcHost);
		strcpy(g_xETHSocket[ulIndex].cDestinationPort, pcPort);
		g_xETHSocket[ulIndex].bUsed = true;
		g_xETHSocket[ulIndex].bStatus = false;
		return ulIndex;
	}    
}

void ETHERNET_SetSocketStatus(unsigned long ulSocketID, bool bStatus)
{
	if (ulSocketID >= g_ulMaxSocketID || g_xETHSocket[ulSocketID].bUsed == false)
		return;
	
	g_xETHSocket[ulSocketID].bStatus = bStatus;
}

bool ETHERNET_GetSocketStatus(unsigned long ulSocketID)
{
	if (ulSocketID >= g_ulMaxSocketID || g_xETHSocket[ulSocketID].bUsed == false)
		return false;
	return g_xETHSocket[ulSocketID].bStatus;
}

void ETHERNET_ReleaseSocket(unsigned long ulSocketID)
{
    if (ulSocketID >= ETH_MAX_SOCKET_COUNT)
		return;
	
	g_xETHSocket[ulSocketID].bUsed = false;
	g_xETHSocket[ulSocketID].bStatus = false;
	memset(g_xETHSocket[ulSocketID].cMode, 0, sizeof(g_xETHSocket[ulSocketID].cMode));
	memset(g_xETHSocket[ulSocketID].cDestinationHost, 0, sizeof(g_xETHSocket[ulSocketID].cDestinationHost));
	memset(g_xETHSocket[ulSocketID].cDestinationPort, 0, sizeof(g_xETHSocket[ulSocketID].cDestinationPort));
}
  
static void ETHERNET_ReconnectTask(void *pvParameter)
{
	while (1)
	{
		int i;
		static bool bIsReconnect = false;
		static unsigned long u32SendStartTick = 0;
		unsigned char ucrepeat = 20;
		
		for (i = 0;i < 4; i++)
		{
			if(g_xETHSocket[i].bUsed == true)
			{
				if(g_xETHSocket[i].bStatus == false)
				{		
					if(((dhcp_mode)get_dhcp_mode()) == NETINFO_DHCP)
					{
						wizchip_initialize(NULL, NULL);
						wizchip_dhcp_init();
						ucrepeat = 20;
						while(ucrepeat--) 
						{   
							if(DHCP_run() == DHCP_IP_LEASED)
							{
								if (pFuncMQTTReconnectHandler != NULL)
								{
									pFuncMQTTReconnectHandler(i);
									g_xETHSocket[i].bStatus = true;
									vTaskDelay(50);
									break;		
								}							
							}
							vTaskDelay(50);
						}
						// if(ucrepeat != 0)
						// 	hw_ping_init();
					}
					else 
					{
						wizchip_initialize(NULL, NULL);
						ctlnetwork(CN_SET_NETINFO, &gWIZNETINFO);
						vTaskDelay(5000);
						if(pFuncMQTTReconnectHandler != NULL)
							pFuncMQTTReconnectHandler(i);
						g_xETHSocket[i].bStatus = true;
						// hw_ping_init();
						vTaskDelay(5000);
					    
					}
				} 
				else if(MQTT_GetStatus(i) == false)
				{		
					if(bIsReconnect == false)
					{
						bIsReconnect = true;
						u32SendStartTick = xTaskGetTickCount();
						g_xETHSocket[i].bStatus = false;
					}
					else
					{
						if(pFuncMQTTReconnectHandler != NULL)
							pFuncMQTTReconnectHandler(i);
					}
				}
				if(abs((long long)xTaskGetTickCount() - (long long)u32SendStartTick) > 10 * 60 * 1000.0f)
				{ 
					bIsReconnect = false;
				}	
			}
		}
		vTaskDelay(200);
	}
}

void if_eth_task_init(void)
{
	xTaskCreate(ETHERNET_ReconnectTask, "ETHERNET Reconnect ", ETHERNET_RECONNECT_TASK_STK, NULL, tskIDLE_PRIORITY + 3, NULL);
}

void setEthSocketStatus(unsigned long socket, bool status)
{
	g_xETHSocket[socket].bStatus = status;
}

void setMQTTStatusChangned(unsigned long socket, bool status)
{
    // printf("addr : %lx\r\n", (unsigned long)(*pFuncMQTTStatusChangedHandler));
	if(pFuncMQTTStatusChangedHandler != NULL)
		pFuncMQTTStatusChangedHandler(socket, status);
}

void check_eth_ping(unsigned long socket)
{
	unsigned char tmp = 0;
	static int timeout = 0;
	char *gw = NULL;
	gw = get_gw_ip();
	setSLPIPR((uint8_t *)gw);            // 目标IP设置：PING对象IP
    setSLCR(0x01);                      // 发送PING请求
	vTaskDelay(2 / portTICK_PERIOD_MS);
	// if(get_eth_int_gpio_status())
	{
		tmp = getSLIR();	
		// printf("tmp %d\r\n",tmp);
		switch(tmp)
		{
			case 0x01:
			    timeout = 0;
				setSLIR(0x01);
				g_xETHSocket[socket].bStatus = true;
			break;
			case 0x04:
				setSLIR(0x04);
				if(timeout++ >= 5)
				{
					timeout = 0;
					g_xETHSocket[socket].bStatus = false;
					setMQTTStatusChangned(socket, false);
					printf("对端PING回应超时\r\n");
				}
			break;
			default:
				if(timeout++ >= 200)
				{
					timeout = 0;
					g_xETHSocket[socket].bStatus = false;
					setMQTTStatusChangned(socket, false);
					printf("对端PING回应超时\r\n");
				}
			break;
		}
	}
}

void check_phy(unsigned long socket)
{
	unsigned char tmp = 0;
	static int timeout = 0;
	// 判断网线是否拔出
	tmp = getPHYSR();
	if((tmp & 0x86) == 0x86 && tmp != 0xff)
	{
		printf("tmp..... %x\r\n",tmp);
		if(timeout++ > 10)
		{
			timeout = 0;
			g_xETHSocket[socket].bStatus = false;
			setMQTTStatusChangned(socket, false);
		}
	}
	else
	{
		timeout = 0;
		g_xETHSocket[socket].bStatus = true;
	}
	vTaskDelay(2 / portTICK_PERIOD_MS);
}