#ifndef CELLUAR_COMM_H
#define CELLUAR_COMM_H
/* Includes ------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "celluar_config.h"
#include "celluar_queue.h"
#include "common.h"

#include "freertos/FreeRTOS.h"
#include "freertos/task.h"
#include "freertos/semphr.h"
#include "freertos/queue.h"

/* Exported macros -----------------------------------------------------------*/
#define CELLUAR_WAITTIME_500MS				500
#define CELLUAR_WAITTIME_1S					1000
#define CELLUAR_WAITTIME_2S					2000
#define CELLUAR_WAITTIME_5S					5000
#define CELLUAR_WAITTIME_10S				10000
#define CELLUAR_WAITTIME_30S				30000
#define CELLUAR_WAITTIME_1M					60000
#define CELLUAR_WAITTIME_2M					120000
#define CELLUAR_WAITTIME_3M					180000

#define CELLUAR_TRYCOUNT_30					30
#define CELLUAR_TRYCOUNT_60					60
#define CELLUAR_TRYCOUNT_90					90

#define CELLUAR_APN_COUNT					6					//APNList数量
#define CELLUAR_NTP_SERVER_COUNT			10

#define CELLUAR_TCP_UDP_MAX_SIZE			1460

#define CELLUAR_HTTP_RESPONSE_HEAD_SIZE		20

/* Exported types ------------------------------------------------------------*/
typedef enum
{
	Celluar_Undefined	= 0,			//Undefined
	Celluar_SIMCom 		= 1,			//SIMCom
	Celluar_Quectel		= 2				//Quectel	
} Celluar_Module;

//Status type
typedef enum
{
	Celluar_OK 			= 0,
	Celluar_Error 		= 1,
	Celluar_Timeout 	= 2
} Celluar_ErrorStatus;

//Celluar Parameter
typedef struct
{
	//Celluar module type
	Celluar_Module				eModule;										//蜂窝模块类型
	//状态
	bool 						bIsPowerOn;										//是否开机
	bool 						bHasSIMCard;									//是否有卡
	bool 						bHasNetwork;									//是否有网
	bool						bWaiting;										//是否在发送指令
	unsigned long 				ulCurrentAPNIndex;								//当前成功APN
	//消息
	Celluar_Queue				xReceiveQueue;									//指令消息接收队列
	//信号量
	SemaphoreHandle_t 			xCommandSemHandle;								//二值信号量，用于命令发送
	SemaphoreHandle_t 			xReceivedSemHandle;								//二值信号量，用于接收消息
	SemaphoreHandle_t 			xResponseSemHandle;								//二值信号量，接收消息后使用
} Celluar;

/* Extern variables ----------------------------------------------------------*/
#if CELLUAR_TCPIP_ENABLE > 0
extern char *g_pcAPNList[CELLUAR_APN_COUNT];
#endif
#if CELLUAR_NTP_ENABLE > 0
extern char *g_pcNTPServerList[CELLUAR_NTP_SERVER_COUNT];
#endif

/* Exported functions --------------------------------------------------------*/
void 				Celluar_Init(void);
void 				Celluar_Restart(void);
bool				Celluar_PowerOnCheckAT(void);
bool 				Celluar_PowerOffCheckAT(void);
bool 				Celluar_PowerOnCheckGPIO(void);
bool 				Celluar_PowerOffCheckGPIO(void);
bool 				Celluar_IsPowerOnWithAT(void);
bool                Celluar_IsSoftReboot(void);
void				Celluar_Take(void);
void				Celluar_Give(void);
bool				Celluar_IsNetStatusOK(void);
bool				Celluar_HasSIMCard(void);
bool				Celluar_IsNetAttached(void);
bool				Celluar_IsPowerOn(void);
Celluar_ErrorStatus Celluar_SendCommandAndWaitForResponse(char *pcCommand, unsigned long ulCommandLen, char *pcDesiredResponse, char *pcResponseData, unsigned long ulResponseBufferSize, unsigned long *pulResponseLen, unsigned long ulWaitTime);
void 				Celluar_SetFunc(Celluar_Module xModule);
Celluar_ErrorStatus	Celluar_GetModuleInfo(char *pcModuleInfo, unsigned long ulModuleBufferSize);
Celluar_ErrorStatus Celluar_SetEchoMode(bool bStatus);

/* Celluar Function Pointer -------------------------------------------------*/
//函数指针
extern void 				(*pFuncCelluar_Poll)(void);												//轮询事件函数
extern void 				(*pFuncCelluar_BasicSettings)(void);
#if CELLUAR_TCPIP_ENABLE > 0
extern void					(*pFuncCelluar_TCPIPInit)(unsigned long *);								//网络状态初始化检测
extern Celluar_ErrorStatus 	(*pFuncCelluar_Connect)(unsigned long);									//网络连接
extern Celluar_ErrorStatus 	(*pFuncCelluar_Send)(unsigned long, unsigned char *, unsigned long);	//数据发送
extern Celluar_ErrorStatus (*pFuncCelluar_GetSocketStatus)(unsigned long);							//检查网络状态
extern Celluar_ErrorStatus (*pFuncCelluar_Disconnect)(unsigned long);								//断开网络连接
#if CELLUAR_USE_SSL > 0
extern Celluar_ErrorStatus (*pFuncCelluar_SetSSL)(bool);											//Set SSL
#endif
#endif
#if CELLUAR_TCPIP_ENABLE > 0 || CELLUAR_SMS_ENABLE > 0
extern Celluar_ErrorStatus (*pFuncCelluar_GetCCID)(char *, unsigned long);							//获取SIM卡CCID
extern Celluar_ErrorStatus (*pFuncCelluar_GetSignal)(unsigned char *, unsigned char *);				//获取信号
#endif
#if CELLUAR_GNSS_ENABLE > 0
extern Celluar_ErrorStatus (*pFuncCelluar_GetGPS)(double *, double *);								//获取GPS
extern Celluar_ErrorStatus (*pFuncCelluar_GetCellArrayInfo)(unsigned long *, unsigned long *, 
							unsigned long *, unsigned long *, unsigned long *, unsigned long *);	//获取基站信息
#endif
#if CELLUAR_NTP_ENABLE > 0
extern bool (*pFuncCelluar_SyncNetworkTime)(Calendar *);											//同步网络时间
#endif
#if CELLUAR_HTTP_ENABLE > 0
extern bool (*pFuncCelluar_HTTPPost)(char *, unsigned char *, unsigned long , 
													unsigned char *, unsigned long, unsigned long *, unsigned long *); 		//Post
extern bool (*pFuncCelluar_HTTPGet)(char *, unsigned char *, unsigned long, unsigned long *, unsigned long *);				//Get
extern Celluar_ErrorStatus (*pFuncCelluar_ReadHTTPServerResponse)(unsigned long, unsigned long, unsigned char *, unsigned long, unsigned long *);
extern Celluar_ErrorStatus (*pFuncCelluar_TerminateHTTPService)(void);
#endif

#endif


